# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
from tempfile import mkdtemp
from six.moves.urllib.parse import urlparse
import hashlib

# project
from kiwi.mount_manager import MountManager
from kiwi.path import Path

from kiwi.exceptions import (
    KiwiUriStyleUnknown,
    KiwiUriTypeUnknown
)


class Uri(object):
    """
    Normalize url types available in a kiwi configuration into
    standard mime types.

    Attributes

    * :attr:`repo_type`
        Repository type name. Only needed if the uri
        is not enough to determine the repository type
        e.g for yast2 vs. rpm-md obs repositories

    * :attr:`uri`
        URI, repository location, file

    * :attr:`repo_type`
        repository type name, rpm-dir, rpm-md, yast2

    * :attr:`mount_stack`
        list of mounted locations

    * :attr:`remote_uri_types`
        dictionary of remote uri type names

    * :attr:`local_uri_type`
        dictionary of local uri type names
    """
    def __init__(self, uri, repo_type=None):
        self.repo_type = repo_type
        self.uri = uri
        self.mount_stack = []

        self.remote_uri_types = {
            'http': True,
            'https': True,
            'ftp': True,
            'obs': True,
            'ibs': True
        }
        self.local_uri_type = {
            'iso': True,
            'dir': True,
            'file': True,
            'suse': True
        }

    def translate(self):
        """
        Translate repository location according to their URI type

        Depending on the URI type the provided location needs to
        be adapted e.g loop mounted in case of an ISO or updated
        by the service URL in case of an open buildservice project
        name
        """
        uri = urlparse(self.uri)
        if not uri.scheme:
            raise KiwiUriStyleUnknown(
                'URI scheme not detected {uri}'.format(uri=self.uri)
            )

        if uri.scheme == 'obs' and self.repo_type == 'yast2':
            return self._obs_distribution(
                ''.join([uri.netloc, uri.path])
            )
        elif uri.scheme == 'obs':
            return self._obs_project(
                ''.join([uri.netloc, uri.path])
            )
        elif uri.scheme == 'ibs':
            return self._ibs_project(
                ''.join([uri.netloc, uri.path])
            )
        elif uri.scheme == 'dir':
            return self._local_path(uri.path)
        elif uri.scheme == 'file':
            return self._local_path(uri.path)
        elif uri.scheme == 'iso':
            return self._iso_mount_path(uri.path)
        elif uri.scheme == 'suse':
            return self._suse_buildservice_path(
                ''.join([uri.netloc, uri.path])
            )
        elif uri.scheme == 'http' or uri.scheme == 'ftp':
            return ''.join([uri.scheme, '://', uri.netloc, uri.path])
        else:
            raise KiwiUriStyleUnknown(
                'URI schema %s not supported' % self.uri
            )

    def credentials_file_name(self):
        """
        Filename to store repository credentials
        """
        uri = urlparse(self.uri)
        # initialize query with default credentials file name.
        # The information will be overwritten if the uri contains
        # a parameter query with a credentials parameter
        query = {'credentials': 'kiwiRepoCredentials'}

        if uri.query:
            query = dict(params.split('=') for params in uri.query.split('&'))

        return query['credentials']

    def alias(self):
        """
        Create hexdigest from URI as alias

        If the repository definition from the XML description does
        not provide an alias, kiwi creates one for you. However it's
        better to assign a human readable alias in the XML
        configuration

        :return: alias name as hexdigest
        :rtype: string
        """
        return hashlib.md5(self.uri.encode()).hexdigest()

    def is_remote(self):
        """
        Check if URI is a remote or local location

        :rtype: bool
        """
        uri = urlparse(self.uri)
        if not uri.scheme:
            raise KiwiUriStyleUnknown(
                'URI scheme not detected %s' % self.uri
            )
        if uri.scheme in self.remote_uri_types:
            return True
        else:
            if uri.scheme in self.local_uri_type:
                return False
            else:
                raise KiwiUriTypeUnknown(
                    'URI type %s unknown' % uri.scheme
                )

    def _iso_mount_path(self, path):
        # The prefix name 'kiwi_iso_mount' has a meaning here because the
        # zypper repository manager looks up iso mount paths by its repo
        # source name
        iso_mount_path = mkdtemp(prefix='kiwi_iso_mount.')
        iso_mount = MountManager(
            device=path, mountpoint=iso_mount_path
        )
        self.mount_stack.append(iso_mount)
        iso_mount.mount()
        return iso_mount.mountpoint

    def _local_path(self, path):
        return os.path.normpath(path)

    def _obs_project(self, name):
        return ''.join(['http://download.opensuse.org/repositories/', name])

    def _ibs_project(self, name):
        ibs_project = 'http://download.suse.de/ibs/'
        return ibs_project + name.replace(':', ':/')

    def _obs_distribution(self, name):
        if name == 'openSUSE:Factory/standard':
            # special handling for SUSE factory repo
            obs_distribution = \
                'http://download.opensuse.org/tumbleweed/repo/oss'
        else:
            obs_distribution = ''.join(
                ['http://download.opensuse.org/distribution/', name]
            )
        return obs_distribution

    def _suse_buildservice_path(self, name):
        """
        Special to openSUSE buildservice. If the buildservice builds
        the image it arranges the repos for each build in a special
        environment, the so called build worker.
        """
        return self._local_path(
            '/usr/src/packages/SOURCES/repos/' + name
        )

    def __del__(self):
        for mount in reversed(self.mount_stack):
            if mount.is_mounted():
                if mount.umount():
                    Path.wipe(mount.mountpoint)



from mock import patch
from mock import call
import mock

from .test_helper import raises

from kiwi.exceptions import (
    KiwiMountKernelFileSystemsError,
    KiwiMountSharedDirectoryError,
    KiwiSetupIntermediateConfigError
)

from kiwi.system.root_bind import RootBind


class TestRootBind(object):
    def setup(self):
        root = mock.Mock()
        root.root_dir = 'root-dir'
        self.bind_root = RootBind(root)

        # stub config files and bind locations
        self.bind_root.config_files = ['/foo']
        self.bind_root.bind_locations = ['/proc']

        # stub files/dirs and mountpoints to cleanup
        self.mount_manager = mock.Mock()
        self.bind_root.cleanup_files = ['/foo.kiwi']
        self.bind_root.mount_stack = [self.mount_manager]
        self.bind_root.dir_stack = ['/mountpoint']

    @raises(KiwiMountKernelFileSystemsError)
    @patch('kiwi.system.root_bind.MountManager.bind_mount')
    @patch('kiwi.system.root_bind.RootBind.cleanup')
    @patch('os.path.exists')
    def test_kernel_file_systems_raises_error(
        self, mock_exists, mock_cleanup, mock_mount
    ):
        mock_exists.return_value = True
        mock_mount.side_effect = KiwiMountKernelFileSystemsError(
            'mount-error'
        )
        self.bind_root.mount_kernel_file_systems()
        mock.cleanup.assert_called_once_with()

    @raises(KiwiMountSharedDirectoryError)
    @patch('kiwi.system.root_bind.MountManager.bind_mount')
    @patch('kiwi.system.root_bind.Path.create')
    @patch('kiwi.system.root_bind.RootBind.cleanup')
    def test_shared_directory_raises_error(
        self, mock_cleanup, mock_path, mock_mount
    ):
        mock_mount.side_effect = KiwiMountSharedDirectoryError(
            'mount-error'
        )
        self.bind_root.mount_shared_directory()
        mock.cleanup.assert_called_once_with()

    @raises(KiwiSetupIntermediateConfigError)
    @patch('kiwi.command.Command.run')
    @patch('kiwi.system.root_bind.RootBind.cleanup')
    @patch('os.path.exists')
    def test_intermediate_config_raises_error(
        self, mock_exists, mock_cleanup, mock_command
    ):
        mock_exists.return_value = True
        mock_command.side_effect = KiwiSetupIntermediateConfigError(
            'config-error'
        )
        self.bind_root.setup_intermediate_config()
        mock.cleanup.assert_called_once_with()

    @patch('kiwi.system.root_bind.MountManager')
    def test_mount_kernel_file_systems(self, mock_mount):
        shared_mount = mock.Mock()
        mock_mount.return_value = shared_mount
        self.bind_root.mount_kernel_file_systems()
        mock_mount.assert_called_once_with(
            device='/proc', mountpoint='root-dir/proc'
        )
        shared_mount.bind_mount.assert_called_once_with()

    @patch('kiwi.system.root_bind.MountManager')
    @patch('kiwi.system.root_bind.Path.create')
    def test_mount_shared_directory(self, mock_path, mock_mount):
        shared_mount = mock.Mock()
        mock_mount.return_value = shared_mount
        self.bind_root.mount_shared_directory()
        mock_path.call_args_list = [
            call('root-dir/var/cache/kiwi'),
            call('/var/cache/kiwi')
        ]
        mock_mount.assert_called_once_with(
            device='/var/cache/kiwi', mountpoint='root-dir/var/cache/kiwi'
        )
        shared_mount.bind_mount.assert_called_once_with()

    @patch('kiwi.command.Command.run')
    @patch('os.path.exists')
    def test_intermediate_config(self, mock_exists, mock_command):
        mock_exists.return_value = True
        self.bind_root.setup_intermediate_config()
        assert mock_command.call_args_list == [
            call([
                'cp', '/foo', 'root-dir/foo.kiwi'
            ]),
            call([
                'ln', '-s', '-f', 'foo.kiwi', 'root-dir/foo'
            ])
        ]

    @patch('kiwi.system.root_bind.MountManager.is_mounted')
    @patch('kiwi.system.root_bind.Command.run')
    @patch('kiwi.system.root_bind.Path.remove_hierarchy')
    @patch('os.path.islink')
    @patch('os.path.exists')
    @patch('shutil.move')
    def test_cleanup(
        self, mock_move, mock_exists, mock_islink, mock_remove_hierarchy,
        mock_command, mock_is_mounted
    ):
        mock_is_mounted.return_value = False
        mock_exists.return_value = True
        mock_islink.return_value = True
        self.bind_root.cleanup()
        self.mount_manager.umount_lazy.assert_called_once_with()
        mock_remove_hierarchy.assert_called_once_with('root-dir/mountpoint')
        mock_command.assert_called_once_with(
            ['rm', '-f', 'root-dir/foo.kiwi', 'root-dir/foo']
        )
        mock_move.assert_called_once_with(
            'root-dir/foo.rpmnew', 'root-dir/foo'
        )

    @patch('os.path.islink')
    @patch('kiwi.logger.log.warning')
    @patch('kiwi.command.Command.run')
    @patch('kiwi.system.root_bind.Path.remove_hierarchy')
    def test_cleanup_continue_on_error(
        self, mock_remove_hierarchy, mock_command, mock_warn, mock_islink
    ):
        mock_islink.return_value = True
        mock_remove_hierarchy.side_effect = Exception
        mock_command.side_effect = Exception
        self.mount_manager.umount_lazy.side_effect = Exception
        self.bind_root.cleanup()
        assert mock_warn.call_args_list == [
            call(
                'Image root directory %s not cleanly umounted: %s',
                'root-dir', ''
            ),
            call(
                'Failed to remove directory %s: %s', '/mountpoint', ''
            ),
            call(
                'Failed to remove intermediate config files: %s', ''
            )
        ]

    @patch('kiwi.logger.log.warning')
    @patch('kiwi.command.Command.run')
    @patch('kiwi.system.root_bind.Path.remove_hierarchy')
    def test_cleanup_nothing_mounted(
        self, mock_remove_hierarchy, mock_command, mock_warn
    ):
        self.mount_manager.is_mounted.return_value = False
        self.mount_manager.mountpoint = '/mountpoint'
        self.bind_root.cleanup()
        mock_warn.assert_called_once_with(
            'Path %s not a mountpoint', '/mountpoint'
        )

    def test_move_to_root(self):
        assert self.bind_root.move_to_root(
            [self.bind_root.root_dir + '/argument']
        ) == ['//argument']



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import collections
from tempfile import NamedTemporaryFile

# project
from kiwi.system.shell import Shell
from kiwi.defaults import Defaults


class Profile(object):
    """
    Create bash readable .profile environment from the XML
    description. The information is used by the kiwi first
    boot code.

    Attributes

    * :attr:`xml_state`
        Instance of XMLState

    * :attr:`dot_profile`
        profile dictionary
    """
    def __init__(self, xml_state):
        self.xml_state = xml_state
        self.dot_profile = {}

        self._image_names_to_profile()
        self._profile_names_to_profile()
        self._packages_marked_for_deletion_to_profile()
        self._type_to_profile()
        self._preferences_to_profile()
        self._systemdisk_to_profile()
        self._strip_to_profile()
        self._machine_to_profile()
        self._oemconfig_to_profile()
        self._drivers_to_profile()

    def add(self, key, value):
        """
        Add key/value pair to profile dictionary

        :param string key: profile key
        :param string value: profile value
        """
        self.dot_profile[key] = value

    def delete(self, key):
        if key in self.dot_profile:
            del self.dot_profile[key]

    def create(self):
        """
        Create bash quoted profile

        :return: profile dump for bash
        :rtype: string
        """
        sorted_profile = collections.OrderedDict(
            sorted(self.dot_profile.items())
        )
        temp_profile = NamedTemporaryFile()
        with open(temp_profile.name, 'w') as profile:
            for key, value in list(sorted_profile.items()):
                if value:
                    profile.write(
                        format(key) + '=' + self._format(value) + '\n'
                    )
        return Shell.quote_key_value_file(temp_profile.name)

    def _oemconfig_to_profile(self):
        # kiwi_oemataraid_scan
        # kiwi_oemvmcp_parmfile
        # kiwi_oemmultipath_scan
        # kiwi_oemswapMB
        # kiwi_oemrootMB
        # kiwi_oemswap
        # kiwi_oempartition_install
        # kiwi_oemdevicefilter
        # kiwi_oemtitle
        # kiwi_oemkboot
        # kiwi_oemreboot
        # kiwi_oemrebootinteractive
        # kiwi_oemshutdown
        # kiwi_oemshutdowninteractive
        # kiwi_oemsilentboot
        # kiwi_oemsilentinstall
        # kiwi_oemsilentverify
        # kiwi_oemskipverify
        # kiwi_oembootwait
        # kiwi_oemunattended
        # kiwi_oemunattended_id
        # kiwi_oemrecovery
        # kiwi_oemrecoveryID
        # kiwi_oemrecoveryPartSize
        # kiwi_oemrecoveryInPlace
        oemconfig = self.xml_state.get_build_type_oemconfig_section()
        if oemconfig:
            self.dot_profile['kiwi_oemataraid_scan'] = \
                self._text(oemconfig.get_oem_ataraid_scan())
            self.dot_profile['kiwi_oemvmcp_parmfile'] = \
                self._text(oemconfig.get_oem_vmcp_parmfile())
            self.dot_profile['kiwi_oemmultipath_scan'] = \
                self._text(oemconfig.get_oem_multipath_scan())
            self.dot_profile['kiwi_oemswapMB'] = \
                self._text(oemconfig.get_oem_swapsize())
            self.dot_profile['kiwi_oemrootMB'] = \
                self._text(oemconfig.get_oem_systemsize())
            self.dot_profile['kiwi_oemswap'] = \
                self._text(oemconfig.get_oem_swap())
            self.dot_profile['kiwi_oempartition_install'] = \
                self._text(oemconfig.get_oem_partition_install())
            self.dot_profile['kiwi_oemdevicefilter'] = \
                self._text(oemconfig.get_oem_device_filter())
            self.dot_profile['kiwi_oemtitle'] = \
                self._text(oemconfig.get_oem_boot_title())
            self.dot_profile['kiwi_oemkboot'] = \
                self._text(oemconfig.get_oem_kiwi_initrd())
            self.dot_profile['kiwi_oemreboot'] = \
                self._text(oemconfig.get_oem_reboot())
            self.dot_profile['kiwi_oemrebootinteractive'] = \
                self._text(oemconfig.get_oem_reboot_interactive())
            self.dot_profile['kiwi_oemshutdown'] = \
                self._text(oemconfig.get_oem_shutdown())
            self.dot_profile['kiwi_oemshutdowninteractive'] = \
                self._text(oemconfig.get_oem_shutdown_interactive())
            self.dot_profile['kiwi_oemsilentboot'] = \
                self._text(oemconfig.get_oem_silent_boot())
            self.dot_profile['kiwi_oemsilentinstall'] = \
                self._text(oemconfig.get_oem_silent_install())
            self.dot_profile['kiwi_oemsilentverify'] = \
                self._text(oemconfig.get_oem_silent_verify())
            self.dot_profile['kiwi_oemskipverify'] = \
                self._text(oemconfig.get_oem_skip_verify())
            self.dot_profile['kiwi_oembootwait'] = \
                self._text(oemconfig.get_oem_bootwait())
            self.dot_profile['kiwi_oemunattended'] = \
                self._text(oemconfig.get_oem_unattended())
            self.dot_profile['kiwi_oemunattended_id'] = \
                self._text(oemconfig.get_oem_unattended_id())
            self.dot_profile['kiwi_oemrecovery'] = \
                self._text(oemconfig.get_oem_recovery())
            self.dot_profile['kiwi_oemrecoveryID'] = \
                self._text(oemconfig.get_oem_recoveryID())
            self.dot_profile['kiwi_oemrecoveryPartSize'] = \
                self._text(oemconfig.get_oem_recovery_part_size())
            self.dot_profile['kiwi_oemrecoveryInPlace'] = \
                self._text(oemconfig.get_oem_inplace_recovery())

    def _drivers_to_profile(self):
        # kiwi_drivers
        self.dot_profile['kiwi_drivers'] = ','.join(
            self.xml_state.get_drivers_list()
        )

    def _machine_to_profile(self):
        # kiwi_xendomain
        machine = self.xml_state.get_build_type_machine_section()
        if machine:
            self.dot_profile['kiwi_xendomain'] = machine.get_domain()

    def _strip_to_profile(self):
        # kiwi_strip_delete
        # kiwi_strip_tools
        # kiwi_strip_libs
        self.dot_profile['kiwi_strip_delete'] = ' '.join(
            self.xml_state.get_strip_files_to_delete()
        )
        self.dot_profile['kiwi_strip_tools'] = ' '.join(
            self.xml_state.get_strip_tools_to_keep()
        )
        self.dot_profile['kiwi_strip_libs'] = ' '.join(
            self.xml_state.get_strip_libraries_to_keep()
        )

    def _systemdisk_to_profile(self):
        # kiwi_lvmgroup
        # kiwi_lvm
        # kiwi_Volume_X
        systemdisk = self.xml_state.get_build_type_system_disk_section()
        if systemdisk:
            self.dot_profile['kiwi_lvmgroup'] = systemdisk.get_name()
            if not self.dot_profile['kiwi_lvmgroup']:
                self.dot_profile['kiwi_lvmgroup'] = \
                    Defaults.get_default_volume_group_name()
            if self.xml_state.get_volume_management():
                self.dot_profile['kiwi_lvm'] = 'true'

            volume_count = 1
            for volume in self.xml_state.get_volumes():
                self.dot_profile['kiwi_Volume_' + format(volume_count)] = '|'.join(
                    [
                        volume.name,
                        'size:all' if volume.fullsize else volume.size,
                        volume.mountpoint or ''
                    ]
                )
                volume_count += 1

    def _preferences_to_profile(self):
        # kiwi_iversion
        # kiwi_showlicense
        # kiwi_keytable
        # kiwi_timezone
        # kiwi_hwclock
        # kiwi_language
        # kiwi_splash_theme
        # kiwi_loader_theme
        for preferences in self.xml_state.get_preferences_sections():
            if 'kiwi_iversion' not in self.dot_profile:
                self.dot_profile['kiwi_iversion'] = \
                    self._text(preferences.get_version())
            if 'kiwi_showlicense' not in self.dot_profile:
                self.dot_profile['kiwi_showlicense'] = \
                    self._text(preferences.get_showlicense())
            if 'kiwi_keytable' not in self.dot_profile:
                self.dot_profile['kiwi_keytable'] = \
                    self._text(preferences.get_keytable())
            if 'kiwi_timezone' not in self.dot_profile:
                self.dot_profile['kiwi_timezone'] = \
                    self._text(preferences.get_timezone())
            if 'kiwi_hwclock' not in self.dot_profile:
                self.dot_profile['kiwi_hwclock'] = \
                    self._text(preferences.get_hwclock())
            if 'kiwi_language' not in self.dot_profile:
                self.dot_profile['kiwi_language'] = \
                    self._text(preferences.get_locale())
            if 'kiwi_splash_theme' not in self.dot_profile:
                self.dot_profile['kiwi_splash_theme'] = \
                    self._text(preferences.get_bootsplash_theme())
            if 'kiwi_loader_theme' not in self.dot_profile:
                self.dot_profile['kiwi_loader_theme'] = \
                    self._text(preferences.get_bootloader_theme())

    def _type_to_profile(self):
        # kiwi_type
        # kiwi_compressed
        # kiwi_boot_timeout
        # kiwi_wwid_wait_timeout
        # kiwi_hybrid
        # kiwi_hybridpersistent
        # kiwi_hybridpersistent_filesystem
        # kiwi_initrd_system
        # kiwi_ramonly
        # kiwi_target_blocksize
        # kiwi_target_removable
        # kiwi_cmdline
        # kiwi_firmware
        # kiwi_bootloader
        # kiwi_bootloader_console
        # kiwi_devicepersistency
        # kiwi_installboot
        # kiwi_bootkernel
        # kiwi_fsmountoptions
        # kiwi_bootprofile
        # kiwi_vga
        # kiwi_btrfs_root_is_snapshot
        type_section = self.xml_state.build_type
        self.dot_profile['kiwi_type'] = \
            type_section.get_image()
        self.dot_profile['kiwi_compressed'] = \
            type_section.get_compressed()
        self.dot_profile['kiwi_boot_timeout'] = \
            type_section.get_boottimeout()
        self.dot_profile['kiwi_wwid_wait_timeout'] = \
            type_section.get_wwid_wait_timeout()
        self.dot_profile['kiwi_hybrid'] = \
            type_section.get_hybrid()
        self.dot_profile['kiwi_hybridpersistent'] = \
            type_section.get_hybridpersistent()
        self.dot_profile['kiwi_hybridpersistent_filesystem'] = \
            type_section.get_hybridpersistent_filesystem()
        self.dot_profile['kiwi_initrd_system'] = \
            type_section.get_initrd_system()
        self.dot_profile['kiwi_ramonly'] = \
            type_section.get_ramonly()
        self.dot_profile['kiwi_target_blocksize'] = \
            type_section.get_target_blocksize()
        self.dot_profile['kiwi_target_removable'] = \
            type_section.get_target_removable()
        self.dot_profile['kiwi_cmdline'] = \
            type_section.get_kernelcmdline()
        self.dot_profile['kiwi_firmware'] = \
            type_section.get_firmware()
        self.dot_profile['kiwi_bootloader'] = \
            type_section.get_bootloader()
        self.dot_profile['kiwi_bootloader_console'] = \
            type_section.get_bootloader_console()
        self.dot_profile['kiwi_btrfs_root_is_snapshot'] = \
            type_section.get_btrfs_root_is_snapshot()
        self.dot_profile['kiwi_gpt_hybrid_mbr'] = \
            type_section.get_gpt_hybrid_mbr()
        self.dot_profile['kiwi_devicepersistency'] = \
            type_section.get_devicepersistency()
        self.dot_profile['kiwi_installboot'] = \
            type_section.get_installboot()
        self.dot_profile['kiwi_bootkernel'] = \
            type_section.get_bootkernel()
        self.dot_profile['kiwi_fsmountoptions'] = \
            type_section.get_fsmountoptions()
        self.dot_profile['kiwi_bootprofile'] = \
            type_section.get_bootprofile()
        self.dot_profile['kiwi_vga'] = \
            type_section.get_vga()

    def _profile_names_to_profile(self):
        # kiwi_profiles
        self.dot_profile['kiwi_profiles'] = ','.join(
            self.xml_state.profiles
        )

    def _packages_marked_for_deletion_to_profile(self):
        # kiwi_delete
        self.dot_profile['kiwi_delete'] = ' '.join(
            self.xml_state.get_to_become_deleted_packages()
        )

    def _image_names_to_profile(self):
        # kiwi_displayname
        # kiwi_cpio_name
        # kiwi_iname
        self.dot_profile['kiwi_iname'] = \
            self.xml_state.xml_data.get_name()

        self.dot_profile['kiwi_displayname'] = \
            self.xml_state.xml_data.get_displayname()
        if not self.dot_profile['kiwi_displayname']:
            self.dot_profile['kiwi_displayname'] = \
                self.dot_profile['kiwi_iname']

        if self.xml_state.get_build_type_name() == 'cpio':
            self.dot_profile['kiwi_cpio_name'] = self.dot_profile['kiwi_iname']

    def _text(self, section_content):
        """
        Helper method to return the text for XML elements of the
        following structure: <section>text</section>. The data
        structure builder will return the text as a list
        """
        if section_content:
            content = section_content[0]
            if content is True:
                return 'true'
            else:
                return content

    def _format(self, value):
        """
        Helper method to format bool profile values in the way
        the boot code expects them
        """
        if value is True:
            return 'true'
        else:
            return format(value)



import sys
import mock

from mock import patch, call

import kiwi

from .test_helper import argv_kiwi_tests

from kiwi.tasks.image_info import ImageInfoTask

from collections import namedtuple


class TestImageInfoTask(object):
    def setup(self):
        sys.argv = [
            sys.argv[0], '--profile', 'vmxFlavour', 'image', 'info',
            '--description', '../data/description',
            '--resolve-package-list'
        ]
        result_type = namedtuple(
            'result_type', [
                'uri', 'installsize_bytes', 'arch', 'version', 'checksum'
            ]
        )
        self.result_info = {
            'image': 'LimeJeOS-openSUSE-13.2',
            'resolved-packages': {
                'packagename': {
                    'version': '0.8.15', 'arch': 'x86_64',
                    'source': 'uri', 'status': 'added_by_dependency_solver',
                    'installsize_bytes': 42
                },
                'filesystem': {
                    'version': '42', 'arch': 'x86_64',
                    'source': 'uri', 'status': 'listed_in_kiwi_description',
                    'installsize_bytes': 0
                }
            }
        }
        self.runtime_checker = mock.Mock()
        kiwi.tasks.base.RuntimeChecker = mock.Mock(
            return_value=self.runtime_checker
        )
        self.solver = mock.MagicMock()
        self.solver.solve = mock.Mock(
            return_value={
                'packagename': result_type(
                    uri='uri', installsize_bytes=42,
                    arch='x86_64', version='0.8.15', checksum='checksum'
                ),
                'filesystem': result_type(
                    uri='uri', installsize_bytes=0,
                    arch='x86_64', version='42', checksum='checksum'
                )
            }
        )
        kiwi.tasks.image_info.Sat = mock.Mock(
            return_value=self.solver
        )

        self.solver_repository = mock.Mock()
        kiwi.tasks.image_info.SolverRepository = mock.Mock(
            return_value=self.solver_repository
        )

        kiwi.tasks.image_info.Help = mock.Mock(
            return_value=mock.Mock()
        )
        self.task = ImageInfoTask()

    def teardown(self):
        sys.argv = argv_kiwi_tests

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['info'] = False
        self.task.command_args['--description'] = '../data/description'
        self.task.command_args['--add-repo'] = []
        self.task.command_args['--obs-repo-internal'] = False
        self.task.command_args['--ignore-repos'] = False
        self.task.command_args['--resolve-package-list'] = False

    @patch('kiwi.tasks.image_info.DataOutput')
    def test_process_image_info(self, mock_out):
        self._init_command_args()
        self.task.command_args['info'] = True
        self.task.process()
        self.runtime_checker.check_repositories_configured.assert_called_once_with()
        mock_out.assert_called_once_with(
            {'image': 'LimeJeOS-openSUSE-13.2'}
        )

    @patch('kiwi.tasks.image_info.DataOutput')
    def test_process_image_info_color_output(self, mock_out):
        self._init_command_args()
        self.task.command_args['info'] = True
        self.task.global_args['--color-output'] = True
        self.task.process()
        mock_out.assert_called_once_with(
            {'image': 'LimeJeOS-openSUSE-13.2'}, style='color'
        )

    @patch('kiwi.tasks.image_info.DataOutput')
    @patch('kiwi.tasks.image_info.SolverRepository')
    @patch('kiwi.tasks.image_info.Uri')
    def test_process_image_info_resolve_package_list(
        self, mock_uri, mock_solver_repo, mock_out
    ):
        self._init_command_args()
        self.task.command_args['info'] = True
        self.task.command_args['--resolve-package-list'] = True
        self.task.process()
        assert self.solver.add_repository.called
        assert mock_uri.call_args_list == [
            call('iso:///image/CDs/dvd.iso', 'yast2'),
            call('obs://Devel:PubCloud:AmazonEC2/SLE_12_GA', 'rpm-md')
        ]
        mock_out.assert_called_once_with(self.result_info)

    @patch('kiwi.xml_state.XMLState.add_repository')
    @patch('kiwi.tasks.image_info.DataOutput')
    def test_process_image_info_add_repo(self, mock_out, mock_state):
        self._init_command_args()
        self.task.command_args['--add-repo'] = [
            'http://example.com,yast2,alias'
        ]
        self.task.process()
        mock_state.assert_called_once_with(
            'http://example.com', 'yast2', 'alias', None
        )

    @patch('kiwi.xml_state.XMLState.translate_obs_to_ibs_repositories')
    @patch('kiwi.tasks.image_info.DataOutput')
    def test_process_image_info_use_ibs_repos(self, mock_out, mock_state):
        self._init_command_args()
        self.task.command_args['--obs-repo-internal'] = True
        self.task.process()
        mock_state.assert_called_once_with()

    @patch('kiwi.xml_state.XMLState.delete_repository_sections')
    @patch('kiwi.tasks.image_info.DataOutput')
    def test_process_image_info_delete_repos(self, mock_out, mock_delete_repos):
        self._init_command_args()
        self.task.command_args['--ignore-repos'] = True
        self.task.process()
        mock_delete_repos.assert_called_once_with()

    def test_process_image_info_help(self):
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['info'] = True
        self.task.process()
        self.task.manual.show.assert_called_once_with(
            'kiwi::image::info'
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.command import Command
from kiwi.defaults import Defaults


class SystemSize(object):
    """
    Provide source tree size information

    Attributes

    * :attr:`source_dir`
        source directory path name
    """
    def __init__(self, source_dir):
        self.source_dir = source_dir

    def customize(self, size, requested_filesystem):
        """
        Increase the sum of all file sizes by an empiric factor

        Each filesystem has some overhead it needs to manage itself.
        Thus the plain data size is always smaller as the size of
        the container which embeds it. This method increases the
        given size by a filesystem specific empiric factor to
        ensure the given data size can be stored in a filesystem
        of the customized size

        :param int size: mbsize to update
        :param string requested_filesystem: filesystem name

        :return: mbytes
        :rtype: int
        """
        if requested_filesystem.startswith('ext'):
            size *= 1.5
            file_count = self.accumulate_files()
            inode_mbytes = \
                file_count * Defaults.get_default_inode_size() / 1048576
            size += 2 * inode_mbytes
        elif requested_filesystem == 'btrfs':
            size *= 1.5
        elif requested_filesystem == 'xfs':
            size *= 1.5

        return int(size)

    def accumulate_mbyte_file_sizes(self):
        """
        Calculate data size of all data in the source tree

        :return: mbytes
        :rtype: int
        """
        du_call = Command.run(
            [
                'du', '-s', '--apparent-size', '--block-size', '1',
                self.source_dir
            ]
        )
        return int(du_call.output.split('\t')[0]) / 1048576

    def accumulate_files(self):
        """
        Calculate sum of all files in the source tree

        :return: number of files
        :rtype: int
        """
        bash_comand = [
            'find', self.source_dir, '|', 'wc', '-l'
        ]
        wc_call = Command.run(
            [
                'bash', '-c', ' '.join(bash_comand)
            ]
        )
        return int(wc_call.output.rstrip('\n'))



from mock import patch
from mock import call

import mock

from .test_helper import raises

from kiwi.exceptions import KiwiBootLoaderZiplInstallError

from kiwi.bootloader.install.zipl import BootLoaderInstallZipl


class TestBootLoaderInstallZipl(object):
    @patch('kiwi.bootloader.install.zipl.MountManager')
    def setup(self, mock_mount):
        custom_args = {
            'boot_device': '/dev/mapper/loop0p1'
        }
        boot_mount = mock.Mock()
        boot_mount.device = custom_args['boot_device']
        boot_mount.mountpoint = 'tmp_boot'

        mock_mount.return_value = boot_mount

        device_provider = mock.Mock()
        device_provider.get_device = mock.Mock(
            return_value='/dev/some-device'
        )
        self.bootloader = BootLoaderInstallZipl(
            'root_dir', device_provider, custom_args
        )

    @raises(KiwiBootLoaderZiplInstallError)
    def test_post_init_missing_boot_mount_path(self):
        self.bootloader.post_init(None)

    def test_install_required(self):
        assert self.bootloader.install_required() is True

    @patch('kiwi.bootloader.install.zipl.Command.run')
    def test_install(self, mock_command):
        self.bootloader.install()
        self.bootloader.boot_mount.mount.assert_called_once_with()
        mock_command.call_args_list == [
            call(
                ['bash', '-c', 'cd tmpdir && zipl -V -c tmpdir/config -m menu']
            ),
            call(
                ['umount', 'tmpdir']
            )
        ]

    def test_destructor(self):
        self.bootloader.__del__()
        self.bootloader.boot_mount.umount.assert_called_once_with()



from mock import patch
from mock import call
import mock

from .test_helper import raises

from kiwi.xml_state import XMLState
from kiwi.xml_description import XMLDescription
from kiwi.exceptions import KiwiBootLoaderTargetError
from kiwi.bootloader.config.base import BootLoaderConfigBase


class TestBootLoaderConfigBase(object):
    def setup(self):
        description = XMLDescription(
            '../data/example_config.xml'
        )
        self.state = XMLState(
            description.load()
        )
        self.bootloader = BootLoaderConfigBase(
            self.state, 'root_dir'
        )

    @raises(NotImplementedError)
    def test_write(self):
        self.bootloader.write()

    @raises(NotImplementedError)
    def test_setup_disk_image_config(self):
        self.bootloader.setup_disk_image_config(
            'boot_uuid', 'root_uuid', 'hypervisor', 'kernel', 'initrd'
        )

    @raises(NotImplementedError)
    def test_setup_install_image_config(self):
        self.bootloader.setup_install_image_config(
            'mbrid', 'hypervisor', 'kernel', 'initrd'
        )

    @raises(NotImplementedError)
    def test_setup_live_image_config(self):
        self.bootloader.setup_live_image_config(
            'mbrid', 'hypervisor', 'kernel', 'initrd'
        )

    @raises(NotImplementedError)
    def test_setup_disk_boot_images(self):
        self.bootloader.setup_disk_boot_images('uuid')

    @raises(NotImplementedError)
    def test_setup_install_boot_images(self):
        self.bootloader.setup_install_boot_images('mbrid')

    @raises(NotImplementedError)
    def test_setup_live_boot_images(self):
        self.bootloader.setup_live_boot_images('mbrid')

    @raises(NotImplementedError)
    def test_setup_sysconfig_bootloader(self):
        self.bootloader.setup_sysconfig_bootloader()

    @patch('kiwi.path.Path.create')
    def test_create_efi_path(self, mock_path):
        self.bootloader.create_efi_path()
        mock_path.assert_called_once_with('root_dir/boot/efi/EFI/BOOT')

    @patch('kiwi.path.Path.create')
    def test_create_efi_path_with_prefix(self, mock_path):
        self.bootloader.create_efi_path('')
        mock_path.assert_called_once_with('root_dir//EFI/BOOT')

    def test_get_boot_theme(self):
        assert self.bootloader.get_boot_theme() == 'openSUSE'

    def test_get_boot_timeout_seconds(self):
        assert self.bootloader.get_boot_timeout_seconds() == 10

    @patch('kiwi.xml_parse.type_.get_installprovidefailsafe')
    def test_failsafe_boot_entry_requested(
        self, mock_installprovidefailsafe
    ):
        mock_installprovidefailsafe.return_value = True
        assert self.bootloader.failsafe_boot_entry_requested() is True
        mock_installprovidefailsafe.return_value = False
        assert self.bootloader.failsafe_boot_entry_requested() is False

    def test_get_hypervisor_domain(self):
        assert self.bootloader.get_hypervisor_domain() == 'domU'

    def test_get_boot_cmdline(self):
        assert self.bootloader.get_boot_cmdline() == 'splash'

    @patch('kiwi.xml_parse.type_.get_kernelcmdline')
    def test_get_boot_cmdline_custom_root(self, mock_cmdline):
        mock_cmdline.return_value = 'root=/dev/myroot'
        assert self.bootloader.get_boot_cmdline() == 'root=/dev/myroot'

    @patch('kiwi.xml_parse.type_.get_firmware')
    def test_get_boot_cmdline_firmware_ec2(self, mock_firmware):
        mock_firmware.return_value = 'ec2'
        assert self.bootloader.get_boot_cmdline('uuid') == \
            'splash root=UUID=uuid rw'

    @patch('kiwi.xml_parse.type_.get_initrd_system')
    def test_get_boot_cmdline_initrd_system_is_dracut(self, mock_initrd):
        mock_initrd.return_value = 'dracut'
        assert self.bootloader.get_boot_cmdline('uuid') == \
            'splash root=UUID=uuid rw'

    @patch('kiwi.xml_parse.type_.get_firmware')
    @patch('kiwi.logger.log.warning')
    def test_get_boot_cmdline_firmware_ec2_no_uuid(
        self, mock_log_warn, mock_firmware
    ):
        mock_firmware.return_value = 'ec2'
        self.bootloader.get_boot_cmdline()
        assert mock_log_warn.called

    @patch('kiwi.xml_parse.type_.get_installboot')
    def test_get_install_image_boot_default(self, mock_installboot):
        mock_installboot.return_value = None
        assert self.bootloader.get_install_image_boot_default() == '0'
        assert self.bootloader.get_install_image_boot_default('isolinux') == \
            'Boot_from_Hard_Disk'
        mock_installboot.return_value = 'failsafe-install'
        assert self.bootloader.get_install_image_boot_default() == '2'
        assert self.bootloader.get_install_image_boot_default('isolinux') == \
            'Failsafe_--_Install_Bob'
        mock_installboot.return_value = 'install'
        assert self.bootloader.get_install_image_boot_default() == '1'
        assert self.bootloader.get_install_image_boot_default('isolinux') == \
            'Install_Bob'

    @patch('kiwi.xml_parse.type_.get_installboot')
    @patch('kiwi.xml_parse.type_.get_installprovidefailsafe')
    @patch('kiwi.logger.log.warning')
    def test_get_install_image_boot_default_failsafe_but_no_failsafe_entry(
        self, mock_warn, mock_installprovidefailsafe, mock_installboot
    ):
        mock_installprovidefailsafe.return_value = False
        mock_installboot.return_value = 'failsafe-install'
        assert self.bootloader.get_install_image_boot_default() == '1'
        assert mock_warn.call_args_list == [
            call(
                'Failsafe install requested but failsafe menu entry is disabled'
            ),
            call('Switching to standard install')
        ]

    @raises(KiwiBootLoaderTargetError)
    def test_get_boot_path_raises(self):
        self.bootloader.get_boot_path('foo')

    @raises(KiwiBootLoaderTargetError)
    @patch('kiwi.bootloader.config.base.DiskSetup')
    @patch('kiwi.xml_parse.type_.get_filesystem')
    @patch('kiwi.xml_state.XMLState.get_volumes')
    def test_get_boot_path_btrfs_boot_is_a_volume_error(
        self, mock_volumes, mock_fs, mock_disk_setup
    ):
        volume = mock.Mock()
        volume.name = 'boot'
        mock_volumes.return_value = [volume]
        mock_fs.return_value = 'btrfs'
        disk_setup = mock.Mock()
        disk_setup.need_boot_partition = mock.Mock(
            return_value=False
        )
        mock_disk_setup.return_value = disk_setup
        self.bootloader.get_boot_path()

    @patch('kiwi.bootloader.config.base.DiskSetup')
    @patch('kiwi.xml_parse.type_.get_filesystem')
    @patch('kiwi.xml_parse.type_.get_btrfs_root_is_snapshot')
    @patch('kiwi.xml_state.XMLState.get_volumes')
    def test_get_boot_path_btrfs_no_snapshot(
        self, mock_volumes, mock_snapshot, mock_fs, mock_disk_setup
    ):
        volume = mock.Mock()
        volume.name = 'some-volume'
        mock_volumes.return_value = [volume]
        mock_fs.return_value = 'btrfs'
        mock_snapshot.return_value = False
        disk_setup = mock.Mock()
        disk_setup.need_boot_partition = mock.Mock(
            return_value=False
        )
        mock_disk_setup.return_value = disk_setup
        assert self.bootloader.get_boot_path() == \
            '/@/boot'

    @patch('kiwi.bootloader.config.base.DiskSetup')
    @patch('kiwi.xml_parse.type_.get_filesystem')
    @patch('kiwi.xml_parse.type_.get_btrfs_root_is_snapshot')
    @patch('kiwi.xml_state.XMLState.get_volumes')
    def test_get_boot_path_btrfs_snapshot(
        self, mock_volumes, mock_snapshot, mock_fs, mock_disk_setup
    ):
        volume = mock.Mock()
        volume.name = 'some-volume'
        mock_volumes.return_value = [volume]
        mock_fs.return_value = 'btrfs'
        mock_snapshot.return_value = True
        disk_setup = mock.Mock()
        disk_setup.need_boot_partition = mock.Mock(
            return_value=False
        )
        mock_disk_setup.return_value = disk_setup
        assert self.bootloader.get_boot_path() == \
            '/boot'

    def test_quote_title(self):
        assert self.bootloader.quote_title('aaa bbb [foo]') == 'aaa_bbb_(foo)'

    @patch('kiwi.xml_parse.image.get_displayname')
    def test_get_menu_entry_title(self, mock_displayname):
        mock_displayname.return_value = None
        assert self.bootloader.get_menu_entry_title() == \
            'LimeJeOS-openSUSE-13.2 [ OEM ]'

    @patch('kiwi.xml_parse.image.get_displayname')
    def test_get_menu_entry_title_plain(self, mock_displayname):
        mock_displayname.return_value = None
        assert self.bootloader.get_menu_entry_title(plain=True) == \
            'LimeJeOS-openSUSE-13.2'

    @patch('kiwi.xml_parse.image.get_displayname')
    def test_get_menu_entry_install_title(self, mock_displayname):
        mock_displayname.return_value = None
        assert self.bootloader.get_menu_entry_install_title() == \
            'LimeJeOS-openSUSE-13.2'

    @patch('kiwi.xml_parse.type_.get_vga')
    def test_get_gfxmode_default(self, mock_get_vga):
        mock_get_vga.return_value = None
        assert self.bootloader.get_gfxmode('isolinux') == '800 600'
        assert self.bootloader.get_gfxmode('grub2') == '800x600'

    @patch('kiwi.xml_parse.type_.get_vga')
    def test_get_gfxmode(self, mock_get_vga):
        mock_get_vga.return_value = '0x318'
        assert self.bootloader.get_gfxmode('isolinux') == '1024 768'
        assert self.bootloader.get_gfxmode('grub2') == '1024x768'

    @patch('kiwi.xml_parse.type_.get_vga')
    def test_get_gfxmode_other_loader(self, mock_get_vga):
        mock_get_vga.return_value = '0x318'
        assert self.bootloader.get_gfxmode('some-loader') == \
            mock_get_vga.return_value



from mock import patch

import mock

from .test_helper import raises, patch_open

from kiwi.exceptions import (
    KiwiTargetDirectoryNotFound,
    KiwiBootImageDumpError
)

from kiwi.boot.image.base import BootImageBase


class TestBootImageBase(object):
    @patch('kiwi.boot.image.base.mkdtemp')
    @patch('kiwi.boot.image.base.os.path.exists')
    @patch('platform.machine')
    def setup(self, mock_machine, mock_exists, mock_mkdtemp):
        mock_machine.return_value = 'x86_64'
        self.boot_xml_state = mock.Mock()
        self.xml_state = mock.Mock()
        self.xml_state.xml_data.get_name = mock.Mock(
            return_value='some-image'
        )
        self.xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        self.xml_state.build_type.get_boot = mock.Mock(
            return_value='oemboot/suse-13.2'
        )
        mock_mkdtemp.return_value = 'boot-root-directory'
        mock_exists.return_value = True
        self.boot_image = BootImageBase(
            self.xml_state, 'some-target-dir'
        )
        mock_mkdtemp.assert_called_once_with(
            prefix='kiwi_boot_root.', dir='some-target-dir'
        )

    @raises(KiwiTargetDirectoryNotFound)
    def test_boot_image_raises(self):
        BootImageBase(
            self.xml_state, 'target-dir-does-not-exist', 'some-root-dir'
        )

    @raises(NotImplementedError)
    def test_prepare(self):
        self.boot_image.prepare()

    @raises(NotImplementedError)
    def test_create_initrd(self):
        self.boot_image.create_initrd()

    @raises(KiwiBootImageDumpError)
    @patch_open
    @patch('pickle.dump')
    def test_dump_error(self, mock_dump, mock_open):
        mock_dump.side_effect = Exception
        self.boot_image.dump('filename')

    @patch_open
    @patch('pickle.dump')
    @patch('kiwi.boot.image.base.BootImageBase.disable_cleanup')
    def test_dump(self, mock_disable_cleanup, mock_dump, mock_open):
        context_manager_mock = mock.Mock()
        mock_open.return_value = context_manager_mock
        file_mock = mock.Mock()
        enter_mock = mock.Mock()
        exit_mock = mock.Mock()
        enter_mock.return_value = file_mock
        setattr(context_manager_mock, '__enter__', enter_mock)
        setattr(context_manager_mock, '__exit__', exit_mock)
        self.boot_image.dump('filename')
        mock_open.assert_called_once_with('filename', 'wb')
        mock_dump.assert_called_once_with(self.boot_image, file_mock)
        mock_disable_cleanup.assert_called_once_with()

    def test_disable_cleanup(self):
        self.boot_image.disable_cleanup()
        assert self.boot_image.call_destructor is False

    def test_enable_cleanup(self):
        self.boot_image.enable_cleanup()
        assert self.boot_image.call_destructor is True

    @patch('os.listdir')
    def test_is_prepared(self, mock_listdir):
        mock_listdir.return_value = True
        assert self.boot_image.is_prepared() == mock_listdir.return_value

    @patch('kiwi.boot.image.base.XMLState.copy_strip_sections')
    def test_import_system_description_elements(self, mock_strip):
        self.boot_image.import_system_description_elements()
        assert self.xml_state.copy_displayname.called
        assert self.xml_state.copy_name.called
        assert self.xml_state.copy_repository_sections.called
        assert self.xml_state.copy_drivers_sections.called
        assert mock_strip.called
        assert self.xml_state.copy_preferences_subsections.called
        assert self.xml_state.copy_bootincluded_packages.called
        assert self.xml_state.copy_bootincluded_archives.called
        assert self.xml_state.copy_bootdelete_packages.called
        assert self.xml_state.copy_build_type_attributes.called
        assert self.xml_state.copy_systemdisk_section.called
        assert self.xml_state.copy_machine_section.called
        assert self.xml_state.copy_oemconfig_section.called

    @patch('kiwi.defaults.Defaults.get_boot_image_description_path')
    def test_get_boot_description_directory(self, boot_path):
        boot_path.return_value = 'boot_path'
        assert self.boot_image.get_boot_description_directory() == \
            'boot_path/oemboot/suse-13.2'

    @patch('kiwi.boot.image.base.Path.wipe')
    @patch('os.path.exists')
    def test_destructor(self, mock_path, mock_wipe):
        mock_path.return_value = True
        self.boot_image.__del__()
        mock_wipe.assert_called_once_with('boot-root-directory')



import mock
from mock import patch
from mock import call

from kiwi.system.root_import.docker import RootImportDocker


class TestRootImportDocker(object):
    @patch('os.path.exists')
    def setup(self, mock_path):
        mock_path.return_value = True
        self.docker_import = RootImportDocker(
            'root_dir', 'file:///image.tar.xz'
        )
        assert self.docker_import.image_file == '/image.tar.xz'

    @patch('kiwi.system.root_import.docker.Compress')
    @patch('kiwi.system.root_import.docker.Command.run')
    @patch('kiwi.system.root_import.docker.DataSync')
    @patch('kiwi.system.root_import.docker.mkdtemp')
    def test_sync_data(
        self, mock_mkdtemp, mock_sync, mock_run, mock_compress
    ):
        uncompress = mock.Mock()
        uncompress.uncompressed_filename = 'tmp_uncompressed'
        mock_compress.return_value = uncompress

        tmpdirs = ['kiwi_unpack_dir', 'kiwi_layout_dir']

        def call_mkdtemp(prefix):
            return tmpdirs.pop()

        mock_mkdtemp.side_effect = call_mkdtemp

        sync = mock.Mock()
        mock_sync.return_value = sync

        self.docker_import.sync_data()

        mock_compress.assert_called_once_with('/image.tar.xz')
        uncompress.uncompress.assert_called_once_with(True)

        assert mock_run.call_args_list == [
            call([
                'skopeo', 'copy', 'docker-archive:tmp_uncompressed',
                'oci:kiwi_layout_dir'
            ]),
            call([
                'umoci', 'unpack', '--image',
                'kiwi_layout_dir', 'kiwi_unpack_dir'
            ])
        ]

        mock_sync.assert_called_once_with(
            'kiwi_unpack_dir/rootfs/', 'root_dir/'
        )
        sync.sync_data.assert_called_once_with(
            options=['-a', '-H', '-X', '-A']
        )

    @patch('kiwi.system.root_import.docker.Path.wipe')
    def test_del(self, mock_path):
        self.docker_import.oci_layout_dir = 'layout_dir'
        self.docker_import.oci_unpack_dir = 'unpack_dir'
        self.docker_import.uncompressed_image = 'uncompressed_file'
        self.docker_import.__del__()

        assert mock_path.call_args_list == [
            call('layout_dir'), call('unpack_dir'), call('uncompressed_file')
        ]



"""
usage: kiwicompat -h | --help
       kiwicompat --build=<description> --dest-dir=<destination>
           [--ignore-repos]
           [(--set-repo=<uri> [--set-repoalias=<alias>] [--set-repopriority=<priority>] [--set-repotype=<type>])]
           [(--add-repo=<uri> [--add-repoalias=<alias>] [--add-repopriority=<priority>] [--add-repotype=<type>])]...
           [--type=<image-type>]
           [--logfile=<filename>]
           [--add-profile=<profile-name>...]
           [--debug]
       kiwicompat --prepare=<description> --root=<image-root>
           [--recycle-root]
           [--logfile=<filename>]
           [--add-profile=<profile-name>...]
           [--ignore-repos]
           [(--set-repo=<uri> [--set-repoalias=<alias>] [--set-repopriority=<priority>] [--set-repotype=<type>])]
           [(--add-repo=<uri> [--add-repoalias=<alias>] [--add-repopriority=<priority>] [--add-repotype=<type>])]...
           [--debug]
       kiwicompat --create=<image-root> --dest-dir=<destination>
           [--ignore-repos]
           [(--set-repo=<uri> [--set-repoalias=<alias>] [--set-repopriority=<priority>] [--set-repotype=<type>])]
           [(--add-repo=<uri> [--add-repoalias=<alias>] [--add-repopriority=<priority>] [--add-repotype=<type>])]...
           [--type=<image-type>]
           [--logfile=<filename>]
           [--add-profile=<profile-name>...]
           [--debug]
       kiwicompat --upgrade=<image-root>
           [--add-package=<name>...]
           [--del-package=<name>...]
           [--logfile=<filename>]
           [--add-profile=<profile-name>...]
           [--debug]
       kiwicompat -v | --version

options:
    -p | --prepare
    -c | --create
    -b | --build
    -d | --dest-dir
    -t | --type
    -u | --upgrade
    -l | --logfile
    -v --version
"""
import os
import logging
from docopt import docopt
from docopt import DocoptExit


class Cli(object):
    """
        Compatibility class for old style kiwi calls
    """
    def __init__(self):
        try:
            self.compat_args = docopt(
                __doc__, options_first=True
            )
        except DocoptExit as e:
            message_header = '\n'.join(
                [
                    'The provided legacy kiwi commandline is invalid',
                    'or not supported. Plase check the following usage',
                    'information if you just mistyped the call:'
                ]
            )
            message_footer = '\n'.join(
                [
                    'In case of a correct legacy kiwi command but not',
                    'supported by kiwicompat, please contact us via the',
                    'github issue system at:\n',
                    'https://github.com/SUSE/kiwi/issues'
                ]
            )
            raise NotImplementedError(
                '%s\n\n%s\n\n%s' %
                (message_header, format(e), message_footer)
            )


class Translate(object):
    def __init__(self, arguments):
        self.arguments = arguments

        self.translated = []
        if self.arguments['--version']:
            self.translated.append('--version')
        if self.arguments['--logfile']:
            self.translated.append('--logfile')
            self.translated.append(self.arguments['--logfile'])
        if self.arguments['--debug']:
            self.translated.append('--debug')
        if self.arguments['--type']:
            self.translated.append('--type')
            self.translated.append(self.arguments['--type'])
        if self.arguments['--add-profile']:
            for profile in self.arguments['--add-profile']:
                self.translated.append('--profile')
                self.translated.append(profile)

        if self.arguments['--create']:
            self.create(self.arguments['--create'])
        elif self.arguments['--prepare']:
            self.prepare(self.arguments['--prepare'])
        elif self.arguments['--upgrade']:
            self.upgrade(self.arguments['--upgrade'])
        elif self.arguments['--build']:
            self.build(self.arguments['--build'])

    def build(self, description):
        self.translated.append('system')
        self.translated.append('build')
        self.translated.append('--description')
        self.translated.append(description)
        self.translated.append('--target-dir')
        self.translated.append(self.arguments['--dest-dir'])
        if self.arguments['--ignore-repos']:
            self.translated.append('--ignore-repos')
        self._set_add_repo_arguments()

    def create(self, root):
        # Note:
        # --ignore-repos, --set-repo and --add-repo
        # options are allowed to be specified for compatibility reasons
        # but are not used in the next generation kiwi because the repo
        # information is persistently stored after the prepare step
        # has finished, which is not the case for the legacy kiwi
        # version
        self.translated.append('system')
        self.translated.append('create')
        self.translated.append('--root')
        self.translated.append(root)
        self.translated.append('--target-dir')
        self.translated.append(self.arguments['--dest-dir'])

    def prepare(self, description):
        self.translated.append('system')
        self.translated.append('prepare')
        self.translated.append('--description')
        self.translated.append(description)
        self.translated.append('--root')
        self.translated.append(self.arguments['--root'])
        if self.arguments['--ignore-repos']:
            self.translated.append('--ignore-repos')
        if self.arguments['--recycle-root']:
            self.translated.append('--allow-existing-root')
        self._set_add_repo_arguments()

    def upgrade(self, root):
        self.translated.append('system')
        self.translated.append('update')
        self.translated.append('--root')
        if self.arguments['--add-package']:
            for add_package in self.arguments['--add-package']:
                self.translated.append('--add-package')
                self.translated.append(add_package)
        if self.arguments['--del-package']:
            for del_package in self.arguments['--del-package']:
                self.translated.append('--delete-package')
                self.translated.append(del_package)
        self.translated.append(root)

    def _set_add_repo_arguments(self):
        if self.arguments['--add-repo']:
            for index in range(0, len(self.arguments['--add-repo'])):
                repo_type = None
                repo_alias = None
                repo_prio = None
                try:
                    repo_type = self.arguments['--add-repotype'][index]
                except Exception:
                    pass
                try:
                    repo_alias = self.arguments['--add-repoalias'][index]
                except Exception:
                    pass
                try:
                    repo_prio = self.arguments['--add-repopriority'][index]
                except Exception:
                    pass

                self.translated.append('--add-repo')
                self.translated.append(self._repo_argument(
                    self.arguments['--add-repo'][index],
                    repo_type, repo_alias, repo_prio
                ))

        if self.arguments['--set-repo']:
            self.translated.append('--set-repo')
            self.translated.append(self._repo_argument(
                self.arguments['--set-repo'],
                self.arguments['--set-repotype'],
                self.arguments['--set-repoalias'],
                self.arguments['--set-repopriority']
            ))

    def _repo_argument(self, source, repo_type, alias, priority):
        if not repo_type:
            repo_type = ''
        if not alias:
            alias = ''
        if not priority:
            priority = ''
        return ','.join(
            [source, repo_type, alias, priority]
        )


class Command(object):
    @classmethod
    def execute(self, arguments):
        os.execvp('kiwi-ng', ['kiwi'] + arguments)


def main():
    logging.basicConfig(format='%(message)s', level=logging.INFO)

    try:
        app = Cli()
        arguments = Translate(app.compat_args)
        # logging.info('Calling: kiwi %s', ' '.join(arguments.translated))
        Command.execute(arguments.translated)
    except NotImplementedError as e:
        logging.error('KiwiCompatError: %s', format(e))
    except OSError as e:
        logging.error('KiwiCompatError: %s', format(e))



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from tempfile import mkdtemp
import os

# project
from kiwi.system.root_import.base import RootImportBase
from kiwi.path import Path
from kiwi.utils.sync import DataSync
from kiwi.utils.compress import Compress
from kiwi.command import Command


class RootImportDocker(RootImportBase):
    """
    Implements the base class for importing a root system from
    a docker image compressed tarball file.
    """
    def post_init(self):
        """
        Post initialization method
        """
        self.uncompressed_image = None
        self.oci_unpack_dir = None
        self.oci_layout_dir = None

    def sync_data(self):
        compressor = Compress(self.image_file)
        compressor.uncompress(True)
        self.uncompressed_image = compressor.uncompressed_filename

        self.oci_layout_dir = mkdtemp(prefix='kiwi_layout_dir.')
        self.oci_unpack_dir = mkdtemp(prefix='kiwi_unpack_dir.')

        Command.run([
            'skopeo', 'copy',
            'docker-archive:{0}'.format(self.uncompressed_image),
            'oci:{0}'.format(self.oci_layout_dir)
        ])
        Command.run([
            'umoci', 'unpack', '--image',
            self.oci_layout_dir, self.oci_unpack_dir
        ])

        synchronizer = DataSync(
            os.sep.join([self.oci_unpack_dir, 'rootfs', '']),
            ''.join([self.root_dir, os.sep])
        )
        synchronizer.sync_data(options=['-a', '-H', '-X', '-A'])

    def __del__(self):
        if self.oci_layout_dir:
            Path.wipe(self.oci_layout_dir)
        if self.oci_unpack_dir:
            Path.wipe(self.oci_unpack_dir)
        if self.uncompressed_image:
            Path.wipe(self.uncompressed_image)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
"""
usage: kiwi result list -h | --help
       kiwi result list --target-dir=<directory>
       kiwi result list help

commands:
    list
        list result information from a previous system command

options:
    --target-dir=<directory>
        the target directory as it was used in a system command
"""
import os

# project
from kiwi.tasks.base import CliTask
from kiwi.help import Help
from kiwi.system.result import Result
from kiwi.logger import log


class ResultListTask(CliTask):
    """
    Implements result listing

    Attributes

    * :attr:`manual`
        Instance of Help
    """
    def process(self):
        """
        List result information from a previous system command
        """
        self.manual = Help()
        if self._help():
            return

        result_directory = os.path.normpath(
            self.command_args['--target-dir']
        )
        log.info(
            'Listing results from %s', result_directory
        )
        result = Result.load(
            result_directory + '/kiwi.result'
        )
        result.print_results()

    def _help(self):
        if self.command_args['help']:
            self.manual.show('kiwi::result::list')
        else:
            return False
        return self.manual



from mock import patch
from mock import call

import mock

from .test_helper import raises

from kiwi.exceptions import (
    KiwiBootLoaderGrubInstallError,
    KiwiBootLoaderGrubDataError,
    KiwiBootLoaderGrubPlatformError
)

from kiwi.bootloader.install.grub2 import BootLoaderInstallGrub2

from kiwi.defaults import Defaults


class TestBootLoaderInstallGrub2(object):
    @patch('platform.machine')
    def setup(self, mock_machine):
        mock_machine.return_value = 'x86_64'

        self.firmware = mock.Mock()
        self.firmware.efi_mode = mock.Mock(
            return_value=None
        )

        self.custom_args = {
            'boot_device': '/dev/mapper/loop0p2',
            'root_device': '/dev/mapper/loop0p1',
            'efi_device': '/dev/mapper/loop0p3',
            'prep_device': '/dev/mapper/loop0p2',
            'system_volumes': {'boot/grub2': {
                'volume_options': 'subvol=@/boot/grub2',
                'volume_device': 'device'
            }},
            'firmware': self.firmware,
            'target_removable': None
        }

        self.root_mount = mock.Mock()
        self.root_mount.device = self.custom_args['root_device']
        self.root_mount.mountpoint = 'tmp_root'

        self.volume_mount = mock.Mock()
        self.volume_mount.device = self.custom_args['root_device']
        self.volume_mount.mountpoint = 'tmp_volume'

        self.boot_mount = mock.Mock()
        self.boot_mount.device = self.custom_args['boot_device']
        self.boot_mount.mountpoint = 'tmp_boot'

        self.efi_mount = mock.Mock()
        self.efi_mount.device = self.custom_args['efi_device']
        self.efi_mount.mountpoint = 'tmp_efi'

        self.device_mount = mock.Mock()
        self.device_mount.device = 'devtmpfs'
        self.device_mount.mountpoint = 'dev'

        self.proc_mount = mock.Mock()
        self.proc_mount.device = 'proc'
        self.proc_mount.mountpoint = 'proc'

        self.sysfs_mount = mock.Mock()
        self.sysfs_mount.device = 'sysfs'
        self.sysfs_mount.mountpoint = 'sys'

        # the order of mount manager is reverse the order in the code
        self.mount_managers = [
            self.efi_mount,
            self.sysfs_mount,
            self.proc_mount,
            self.device_mount,
            self.volume_mount,
            self.boot_mount,
            self.root_mount
        ]

        device_provider = mock.Mock()
        device_provider.get_device = mock.Mock(
            return_value='/dev/some-device'
        )

        self.bootloader = BootLoaderInstallGrub2(
            'root_dir', device_provider, self.custom_args
        )

    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    @raises(KiwiBootLoaderGrubInstallError)
    def test_post_init_ppc_no_prep_device(self, mock_grub_path):
        self.bootloader.arch = 'ppc64'
        del self.custom_args['prep_device']
        self.bootloader.install()

    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    @raises(KiwiBootLoaderGrubDataError)
    def test_grub2_bootloader_not_installed(
        self, mock_grub_path, mock_mount_manager, mock_command
    ):
        mock_grub_path.return_value = None
        self.bootloader.arch = 'x86_64'
        self.bootloader.install()

    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    @raises(KiwiBootLoaderGrubPlatformError)
    def test_unsupported_platform(self, mock_grub_path):
        self.bootloader.arch = 'unsupported'
        self.bootloader.install()

    @raises(KiwiBootLoaderGrubInstallError)
    def test_post_init_no_boot_device(self):
        self.bootloader.post_init({})

    @raises(KiwiBootLoaderGrubInstallError)
    def test_post_init_no_root_device(self):
        self.bootloader.post_init({'boot_device': 'a'})

    @raises(KiwiBootLoaderGrubInstallError)
    def test_post_init_secure_boot_no_efi_device(self):
        self.firmware.efi_mode.return_value = 'uefi'
        del self.custom_args['efi_device']
        self.bootloader.post_init(self.custom_args)

    def test_install_required(self):
        assert self.bootloader.install_required() is True

    def test_install_required_ppc_opal(self):
        self.bootloader.arch = 'ppc64'
        self.bootloader.firmware = mock.Mock()
        self.bootloader.firmware.opal_mode = mock.Mock(
            return_value=True
        )
        assert self.bootloader.install_required() is False

    def test_install_required_arm64(self):
        self.bootloader.arch = 'arm64'
        assert self.bootloader.install_required() is False

    @patch('kiwi.bootloader.install.grub2.Path.wipe')
    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    def test_install_with_extra_boot_partition(
        self, mock_grub_path, mock_mount_manager, mock_command, mock_wipe
    ):
        mock_grub_path.return_value = \
            self.root_mount.mountpoint + '/usr/lib/grub2'

        def side_effect(device, mountpoint=None):
            return self.mount_managers.pop()

        mock_mount_manager.side_effect = side_effect

        self.bootloader.install()
        self.bootloader.root_mount.mount.assert_called_once_with()
        self.bootloader.boot_mount.mount.assert_called_once_with()
        mock_wipe.assert_called_once_with(
            'tmp_root/boot/grub2/grubenv'
        )
        mock_command.assert_called_once_with(
            [
                'chroot', 'tmp_root', 'grub2-install', '--skip-fs-probe',
                '--directory', '/usr/lib/grub2/i386-pc',
                '--boot-directory', '/boot',
                '--target', 'i386-pc',
                '--modules', ' '.join(
                    Defaults.get_grub_bios_modules(multiboot=True)
                ),
                '/dev/some-device'
            ]
        )

    @patch('kiwi.bootloader.install.grub2.Path.wipe')
    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    def test_install_ppc_ieee1275(
        self, mock_grub_path, mock_mount_manager, mock_command, mock_wipe
    ):
        mock_grub_path.return_value = \
            self.root_mount.mountpoint + '/usr/lib/grub2'
        self.bootloader.arch = 'ppc64'

        def side_effect(device, mountpoint=None):
            return self.mount_managers.pop()

        mock_mount_manager.side_effect = side_effect

        self.bootloader.install()
        self.bootloader.root_mount.mount.assert_called_once_with()
        self.bootloader.boot_mount.mount.assert_called_once_with()
        mock_wipe.assert_called_once_with(
            'tmp_root/boot/grub2/grubenv'
        )
        mock_command.assert_called_once_with(
            [
                'chroot', 'tmp_root', 'grub2-install', '--skip-fs-probe',
                '--no-nvram', '--directory', '/usr/lib/grub2/powerpc-ieee1275',
                '--boot-directory', '/boot',
                '--target', 'powerpc-ieee1275',
                '--modules', ' '.join(
                    Defaults.get_grub_ofw_modules()
                ),
                self.custom_args['prep_device']
            ]
        )

    @patch('kiwi.bootloader.install.grub2.Path.wipe')
    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    def test_install(
        self, mock_grub_path, mock_mount_manager, mock_command, mock_wipe
    ):
        mock_grub_path.return_value = \
            self.root_mount.mountpoint + '/usr/lib/grub2'
        self.boot_mount.device = self.root_mount.device

        def side_effect(device, mountpoint=None):
            return self.mount_managers.pop()

        mock_mount_manager.side_effect = side_effect
        self.bootloader.target_removable = True

        self.bootloader.install()
        self.root_mount.mount.assert_called_once_with()
        self.volume_mount.mount.assert_called_once_with(
            options=['subvol=@/boot/grub2']
        )
        mock_wipe.assert_called_once_with(
            'tmp_root/boot/grub2/grubenv'
        )
        mock_command.assert_called_once_with(
            [
                'chroot', 'tmp_root', 'grub2-install',
                '--removable', '--skip-fs-probe',
                '--directory', '/usr/lib/grub2/i386-pc',
                '--boot-directory', '/boot',
                '--target', 'i386-pc',
                '--modules', ' '.join(
                    Defaults.get_grub_bios_modules(multiboot=True)
                ),
                '/dev/some-device'
            ])

    @patch('kiwi.bootloader.install.grub2.Path.wipe')
    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    @patch('os.path.exists')
    def test_install_secure_boot(
        self, mock_exists, mock_grub_path, mock_mount_manager,
        mock_command, mock_wipe
    ):
        mock_exists.return_value = True
        mock_grub_path.return_value = \
            self.root_mount.mountpoint + '/usr/lib/grub2'
        self.firmware.efi_mode.return_value = 'uefi'
        self.boot_mount.device = self.root_mount.device

        def side_effect(device, mountpoint=None):
            return self.mount_managers.pop()

        mock_mount_manager.side_effect = side_effect

        self.bootloader.install()

        mock_wipe.assert_called_once_with(
            'tmp_root/boot/grub2/grubenv'
        )
        assert mock_command.call_args_list == [
            call([
                'chroot', 'tmp_root', 'grub2-install', '--skip-fs-probe',
                '--directory', '/usr/lib/grub2/i386-pc',
                '--boot-directory', '/boot',
                '--target', 'i386-pc',
                '--modules', ' '.join(
                    Defaults.get_grub_bios_modules(multiboot=True)
                ),
                '/dev/some-device'
            ]),
            call([
                'cp', '-p', 'tmp_root/usr/sbin/grub2-install',
                'tmp_root/usr/sbin/grub2-install.orig'
            ]),
            call([
                'cp', 'tmp_root/bin/true', 'tmp_root/usr/sbin/grub2-install'
            ]),
            call([
                'chroot', 'tmp_root', 'shim-install', '--removable',
                '/dev/some-device'
            ]),
            call([
                'cp', '-p', 'tmp_root/usr/sbin/grub2-install.orig',
                'tmp_root/usr/sbin/grub2-install'
            ])
        ]
        self.device_mount.bind_mount.assert_called_once_with()
        self.proc_mount.bind_mount.assert_called_once_with()
        self.sysfs_mount.bind_mount.assert_called_once_with()
        self.efi_mount.mount.assert_called_once_with()

    @patch('kiwi.bootloader.install.grub2.Command.run')
    @patch('kiwi.bootloader.install.grub2.MountManager')
    @patch('kiwi.bootloader.install.grub2.Defaults.get_grub_path')
    def test_destructor(self, mock_grub_path, mock_mount_manager, mock_command):
        self.firmware.efi_mode.return_value = 'uefi'

        def side_effect(device, mountpoint=None):
            return self.mount_managers.pop()

        mock_mount_manager.side_effect = side_effect

        self.bootloader.install()
        self.bootloader.__del__()

        self.device_mount.umount.assert_called_once_with()
        self.proc_mount.umount.assert_called_once_with()
        self.sysfs_mount.umount.assert_called_once_with()
        self.efi_mount.umount.assert_called_once_with()
        self.boot_mount.umount.assert_called_once_with()
        self.root_mount.umount.assert_called_once_with()



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import platform
import pickle
from tempfile import mkdtemp

# project
from kiwi.defaults import Defaults
from kiwi.xml_description import XMLDescription
from kiwi.xml_state import XMLState
from kiwi.logger import log
from kiwi.path import Path

from kiwi.exceptions import (
    KiwiTargetDirectoryNotFound,
    KiwiConfigFileNotFound,
    KiwiBootImageDumpError
)


class BootImageBase(object):
    """
    Base class for boot image(initrd) task

    Attributes

    * :attr:`xml_state`
        Instance of XMLState of the system image description

    * :attr:`target_dir`
        target dir to store the initrd

    * :attr:`initrd_filename`
        initrd file name

    * :attr:`temp_boot_root_directory`
        temporary directory to create the initrd system

    * :attr:`boot_xml_state`
        Instance of XMLState of the boot image description

    """
    def __init__(self, xml_state, target_dir, root_dir=None):
        self.xml_state = xml_state
        self.target_dir = target_dir
        self.initrd_filename = None
        self.boot_xml_state = None
        self.setup = None
        self.temp_directories = []
        self.call_destructor = True

        self.boot_root_directory = root_dir
        if not self.boot_root_directory:
            self.boot_root_directory = mkdtemp(
                prefix='kiwi_boot_root.', dir=self.target_dir
            )
            self.temp_directories.append(
                self.boot_root_directory
            )

        if not os.path.exists(target_dir):
            raise KiwiTargetDirectoryNotFound(
                'target directory %s not found' % target_dir
            )

        self.initrd_file_name = ''.join(
            [
                self.target_dir, '/',
                self.xml_state.xml_data.get_name(),
                '.' + platform.machine(),
                '-' + self.xml_state.get_image_version(),
                '.initrd'
            ]
        )

    def dump(self, filename):
        """
        Pickle dump this instance to a file. If the object dump
        is requested the destructor code will also be disabled
        in order to preserve the generated data

        :param string filename: file path name
        """
        try:
            with open(filename, 'wb') as boot_image:
                pickle.dump(self, boot_image)
            self.disable_cleanup()
        except Exception as e:
            raise KiwiBootImageDumpError(
                'Failed to pickle dump boot image: %s' % format(e)
            )

    def disable_cleanup(self):
        self.call_destructor = False

    def enable_cleanup(self):
        self.call_destructor = True

    def prepare(self):
        """
        Prepare new root system to create initrd from. Implementation
        is only needed if there is no other root system available
        """
        raise NotImplementedError

    def create_initrd(self, mbrid=None):
        """
        Implements creation of the initrd

        :param object mbrid: instance of ImageIdentifier
        """
        raise NotImplementedError

    def is_prepared(self):
        """
        Check if initrd system is prepared

        :return: initrd system preparation status
        :rtype: bool
        """
        return os.listdir(self.boot_root_directory)

    def load_boot_xml_description(self):
        """
        Load the boot image description referenced by the system image
        description boot attribute
        """
        log.info('Loading Boot XML description')
        boot_description_directory = self.get_boot_description_directory()
        boot_config_file = boot_description_directory + '/config.xml'
        if not os.path.exists(boot_config_file):
            raise KiwiConfigFileNotFound(
                'no Boot XML description found in %s' %
                boot_description_directory
            )
        boot_description = XMLDescription(
            description=boot_config_file,
            derived_from=self.xml_state.xml_data.description_dir
        )

        boot_image_profile = self.xml_state.build_type.get_bootprofile()
        if not boot_image_profile:
            boot_image_profile = 'default'
        boot_kernel_profile = self.xml_state.build_type.get_bootkernel()
        if not boot_kernel_profile:
            boot_kernel_profile = 'std'

        self.boot_xml_state = XMLState(
            boot_description.load(), [boot_image_profile, boot_kernel_profile]
        )
        log.info('--> loaded %s', boot_config_file)
        if self.boot_xml_state.build_type:
            log.info(
                '--> Selected build type: %s',
                self.boot_xml_state.get_build_type_name()
            )
        if self.boot_xml_state.profiles:
            log.info(
                '--> Selected boot profiles: image: %s, kernel: %s',
                boot_image_profile, boot_kernel_profile
            )

    def import_system_description_elements(self):
        """
        Copy information from the system image relevant to create the
        boot image to the boot image state XML description
        """
        self.xml_state.copy_displayname(
            self.boot_xml_state
        )
        self.xml_state.copy_name(
            self.boot_xml_state
        )
        self.xml_state.copy_repository_sections(
            target_state=self.boot_xml_state,
            wipe=True
        )
        self.xml_state.copy_drivers_sections(
            self.boot_xml_state
        )
        strip_description = XMLDescription(
            Defaults.get_boot_image_strip_file()
        )
        strip_xml_state = XMLState(strip_description.load())
        strip_xml_state.copy_strip_sections(
            self.boot_xml_state
        )
        preferences_subsection_names = [
            'bootloader_theme',
            'bootsplash_theme',
            'locale',
            'packagemanager',
            'rpm_check_signatures',
            'rpm_excludedocs',
            'showlicense'
        ]
        self.xml_state.copy_preferences_subsections(
            preferences_subsection_names, self.boot_xml_state
        )
        self.xml_state.copy_bootincluded_packages(
            self.boot_xml_state
        )
        self.xml_state.copy_bootincluded_archives(
            self.boot_xml_state
        )
        self.xml_state.copy_bootdelete_packages(
            self.boot_xml_state
        )
        type_attributes = [
            'bootkernel',
            'bootloader',
            'bootloader_console',
            'bootprofile',
            'boottimeout',
            'btrfs_root_is_snapshot',
            'gpt_hybrid_mbr',
            'devicepersistency',
            'filesystem',
            'firmware',
            'fsmountoptions',
            'hybrid',
            'hybridpersistent',
            'hybridpersistent_filesystem',
            'initrd_system',
            'installboot',
            'installprovidefailsafe',
            'kernelcmdline',
            'ramonly',
            'target_removable',
            'vga',
            'wwid_wait_timeout'
        ]
        self.xml_state.copy_build_type_attributes(
            type_attributes, self.boot_xml_state
        )
        self.xml_state.copy_systemdisk_section(
            self.boot_xml_state
        )
        self.xml_state.copy_machine_section(
            self.boot_xml_state
        )
        self.xml_state.copy_oemconfig_section(
            self.boot_xml_state
        )

    def get_boot_description_directory(self):
        """
        Provide path to the boot image XML description

        :return: path name
        :rtype: string
        """
        boot_description = self.xml_state.build_type.get_boot()
        if boot_description:
            if not boot_description[0] == '/':
                boot_description = \
                    Defaults.get_boot_image_description_path() + '/' + \
                    boot_description
            return boot_description

    def __del__(self):
        if self.call_destructor:
            log.info('Cleaning up %s instance', type(self).__name__)
            for directory in self.temp_directories:
                if directory and os.path.exists(directory):
                    Path.wipe(directory)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os

# project
from kiwi.system.uri import Uri
from kiwi.exceptions import KiwiRootImportError


class RootImportBase(object):
    """
    Imports the root system from an already packed image.

    * :attr:`root_dir`
        root directory path name

    * :attr:`image_file`
        local image file to import

    * :attr:`tmp_root_dir`
        temporary directory where image_file is extracted
    """
    def __init__(self, root_dir, image_uri):
        uri = Uri(image_uri)
        if uri.is_remote():
            raise KiwiRootImportError(
                'Only local imports are supported'
            )
        else:
            self.image_file = uri.translate()
            if not os.path.exists(self.image_file):
                raise KiwiRootImportError(
                    'Could not stat base image file: {0}'.format(self.image_file)
                )

        self.root_dir = root_dir
        self.post_init()

    def post_init(self):
        """
        Post initalization method

        Implementation in specialized root import class
        """
        pass

    def sync_data(self):
        """
        Synchronizes the root system of `image_file` into the root_dir

        Implementation in specialized root import class
        """
        raise NotImplementedError



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import platform
from collections import OrderedDict

# project
from kiwi.command import Command
from kiwi.defaults import Defaults
from kiwi.path import Path
from kiwi.logger import log

from kiwi.exceptions import (
    KiwiFormatSetupError,
    KiwiResizeRawDiskError
)


class DiskFormatBase(object):
    """
    Base class to create disk formats from a raw disk image

    Attributes

    * :attr:`xml_state`
        Instance of XMLState

    * :attr:`root_dir`
        root directory path name

    * :attr:`arch`
        platform.machine

    * :attr:`target_dir`
        target directory path name

    * :attr:`custom_args`
        list of custom format options

    * :attr:`temp_image_dir`
        temporary manifest directory

    * :attr:`diskname`
        raw disk file path name

    * :attr:`image_format`
        disk format name
    """
    def __init__(self, xml_state, root_dir, target_dir, custom_args=None):
        self.xml_state = xml_state
        self.root_dir = root_dir
        self.arch = platform.machine()
        self.target_dir = target_dir
        self.custom_args = {}
        self.temp_image_dir = None
        self.image_format = None
        self.diskname = self.get_target_name_for_format('raw')

        self.post_init(custom_args)

    def post_init(self, custom_args):
        """
        Post initialization method

        Implementation in specialized disk format class if required

        :param list custom_args: unused
        """
        pass

    def has_raw_disk(self):
        """
        Check if the base raw disk image exists

        :rtype: bool
        """
        return os.path.exists(self.diskname)

    def resize_raw_disk(self, size_bytes):
        """
        Resize raw disk image to specified size. If the request
        would actually shrink the disk an exception is raised.
        If the disk got changed the method returns True, if
        the new size is the same as the current size nothing
        gets resized and the method returns False

        :param int size: new size in bytes
        :rtype: bool
        """
        current_byte_size = os.path.getsize(self.diskname)
        size_bytes = int(size_bytes)
        if size_bytes < current_byte_size:
            raise KiwiResizeRawDiskError(
                'shrinking {0} disk to {1} bytes corrupts the image'.format(
                    self.diskname, size_bytes
                )
            )
        elif size_bytes == current_byte_size:
            return False
        Command.run(
            ['qemu-img', 'resize', self.diskname, format(size_bytes)]
        )
        return True

    def create_image_format(self):
        """
        Create disk format

        Implementation in specialized disk format class required
        """
        raise NotImplementedError

    def get_qemu_option_list(self, custom_args):
        """
        Create list of qemu options from custom_args dict

        :param dict custom_args: arguments

        :return: qemu option list
        :rtype: list
        """
        options = []
        if custom_args:
            ordered_args = OrderedDict(sorted(custom_args.items()))
            for key, value in list(ordered_args.items()):
                options.append('-o')
                options.append(key)
                if value:
                    options.append(value)
        return options

    def get_target_name_for_format(self, format_name):
        """
        Create target file path name for specified format

        :param string format_name: disk format name

        :return: file path name
        :rtype: string
        """
        if format_name != 'raw':
            if format_name not in Defaults.get_disk_format_types():
                raise KiwiFormatSetupError(
                    'unsupported disk format %s' % format_name
                )
        return ''.join(
            [
                self.target_dir, '/',
                self.xml_state.xml_data.get_name(),
                '.' + self.arch,
                '-' + self.xml_state.get_image_version(),
                '.' + format_name
            ]
        )

    def store_to_result(self, result):
        """
        Store result file of the format conversion into the
        provided result instance.

        By default only the converted image file will be stored.
        Subformats which creates additional metadata files needs
        to overwrite this method

        :param object result: Instance of Result
        """
        result.add(
            key='disk_format_image',
            filename=self.get_target_name_for_format(
                self.image_format
            ),
            use_for_bundle=True,
            compress=False,
            shasum=True
        )

    def __del__(self):
        if self.temp_image_dir and os.path.exists(self.temp_image_dir):
            log.info('Cleaning up %s instance', type(self).__name__)
            Path.wipe(self.temp_image_dir)



from mock import patch
from mock import call
import mock
import kiwi

from .test_helper import raises, patch_open

from kiwi.exceptions import KiwiLiveBootImageError

from kiwi.builder.live import LiveImageBuilder


class TestLiveImageBuilder(object):
    @patch('platform.machine')
    def setup(self, mock_machine):
        mock_machine.return_value = 'x86_64'
        self.firmware = mock.Mock()
        self.firmware.efi_mode = mock.Mock(
            return_value='uefi'
        )
        kiwi.builder.live.FirmWare = mock.Mock(
            return_value=self.firmware
        )
        self.setup = mock.Mock()
        kiwi.builder.live.SystemSetup = mock.Mock(
            return_value=self.setup
        )
        self.boot_image_task = mock.Mock()
        self.boot_image_task.boot_root_directory = 'initrd_dir'
        self.boot_image_task.initrd_filename = 'initrd'
        kiwi.builder.live.BootImage = mock.Mock(
            return_value=self.boot_image_task
        )
        self.mbrid = mock.Mock()
        self.mbrid.get_id = mock.Mock(
            return_value='0xffffffff'
        )
        kiwi.builder.live.SystemIdentifier = mock.Mock(
            return_value=self.mbrid
        )
        kiwi.builder.live.Path = mock.Mock()
        self.kernel = mock.Mock()
        self.kernel.get_kernel = mock.Mock()
        self.kernel.get_xen_hypervisor = mock.Mock()
        self.kernel.copy_kernel = mock.Mock()
        self.kernel.copy_xen_hypervisor = mock.Mock()
        kiwi.builder.live.Kernel = mock.Mock(
            return_value=self.kernel
        )
        self.xml_state = mock.Mock()
        self.xml_state.get_fs_mount_option_list = mock.Mock(
            return_value='async'
        )
        self.xml_state.build_type.get_flags = mock.Mock(
            return_value=None
        )
        self.xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        self.xml_state.xml_data.get_name = mock.Mock(
            return_value='result-image'
        )
        self.xml_state.build_type.get_volid = mock.Mock(
            return_value='volid'
        )
        self.xml_state.build_type.get_kernelcmdline = mock.Mock(
            return_value='custom_cmdline'
        )
        self.live_image = LiveImageBuilder(
            self.xml_state, 'target_dir', 'root_dir'
        )
        self.live_image.machine = mock.Mock()
        self.live_image.machine.get_domain = mock.Mock(
            return_value='dom0'
        )
        self.result = mock.Mock()
        self.live_image.result = self.result
        self.live_image.hybrid = True

    @patch('platform.machine')
    def test_init_for_ix86_platform(self, mock_machine):
        xml_state = mock.Mock()
        xml_state.xml_data.get_name = mock.Mock(
            return_value='some-image'
        )
        xml_state.get_image_version = mock.Mock(
            return_value='1.2.3'
        )
        mock_machine.return_value = 'i686'
        live_image = LiveImageBuilder(
            xml_state, 'target_dir', 'root_dir'
        )
        assert live_image.arch == 'ix86'

    @patch('shutil.copy')
    @patch('kiwi.builder.live.mkdtemp')
    @patch('kiwi.builder.live.Command.run')
    @patch('kiwi.builder.live.Iso.create_hybrid')
    @patch('kiwi.builder.live.FileSystem')
    @patch('kiwi.builder.live.FileSystemIsoFs')
    @patch('kiwi.builder.live.BootLoaderConfig')
    @patch('kiwi.builder.live.SystemSize')
    @patch_open
    def test_create_overlay_structure(
        self, mock_open, mock_size, mock_bootloader, mock_isofs, mock_fs,
        mock_hybrid, mock_command, mock_dtemp, mock_copy
    ):
        tmpdir_name = ['temp-squashfs', 'temp_media_dir']

        def side_effect(prefix, dir):
            return tmpdir_name.pop()

        mock_dtemp.side_effect = side_effect
        context_manager_mock = mock.Mock()
        mock_open.return_value = context_manager_mock
        file_mock = mock.Mock()
        enter_mock = mock.Mock()
        exit_mock = mock.Mock()
        enter_mock.return_value = file_mock
        setattr(context_manager_mock, '__enter__', enter_mock)
        setattr(context_manager_mock, '__exit__', exit_mock)
        self.live_image.live_type = 'overlay'
        live_type_image = mock.Mock()
        mock_fs.return_value = live_type_image
        bootloader = mock.Mock()
        mock_bootloader.return_value = bootloader
        iso_image = mock.Mock()
        iso_image.create_on_file.return_value = 'offset'
        mock_isofs.return_value = iso_image
        rootsize = mock.Mock()
        rootsize.accumulate_mbyte_file_sizes = mock.Mock(
            return_value=8192
        )
        mock_size.return_value = rootsize
        self.setup.export_rpm_package_verification.return_value = '.verified'
        self.setup.export_rpm_package_list.return_value = '.packages'

        self.live_image.create()

        self.live_image.boot_image_task.prepare.assert_called_once_with()
        self.setup.export_modprobe_setup.assert_called_once_with(
            'initrd_dir'
        )
        mock_fs.assert_called_once_with(
            custom_args={'mount_options': 'async'},
            device_provider=None, name='squashfs', root_dir='root_dir'
        )
        live_type_image.create_on_file.assert_called_once_with(
            'target_dir/result-image-read-only.x86_64-1.2.3'
        )
        assert mock_command.call_args_list[0] == call(
            [
                'mv', 'target_dir/result-image-read-only.x86_64-1.2.3',
                'temp_media_dir'
            ]
        )
        mock_open.assert_called_once_with(
            'temp_media_dir/config.isoclient', 'w'
        )
        assert file_mock.write.call_args_list == [
            call('IMAGE="loop;result-image.x86_64;1.2.3"\n'),
            call('UNIONFS_CONFIG="tmpfs,loop,overlay"\n')
        ]

        assert mock_bootloader.call_args_list[0] == call(
            'isolinux', self.xml_state, 'temp_media_dir'
        )
        assert bootloader.setup_live_boot_images.call_args_list[0] == call(
            lookup_path=self.live_image.boot_image_task.boot_root_directory,
            mbrid=None
        )
        assert bootloader.setup_live_image_config.call_args_list[0] == call(
            mbrid=None
        )
        assert bootloader.write.call_args_list[0] == call()
        mock_copy.assert_called_once_with(
            'temp_media_dir/boot/grub2/grub.cfg', 'temp_media_dir/EFI/BOOT'
        )

        assert mock_bootloader.call_args_list[1] == call(
            'grub2', self.xml_state, 'temp_media_dir'
        )
        assert bootloader.setup_live_boot_images.call_args_list[1] == call(
            lookup_path=self.live_image.boot_image_task.boot_root_directory,
            mbrid=self.mbrid
        )
        assert bootloader.setup_live_image_config.call_args_list[1] == call(
            mbrid=self.mbrid
        )
        assert bootloader.write.call_args_list[1] == call()

        self.boot_image_task.create_initrd.assert_called_once_with(
            self.mbrid
        )
        self.kernel.copy_kernel.assert_called_once_with(
            'temp_media_dir/boot/x86_64/loader', '/linux'
        )
        self.kernel.copy_xen_hypervisor.assert_called_once_with(
            'temp_media_dir/boot/x86_64/loader', '/xen.gz'
        )
        assert mock_command.call_args_list[1] == call(
            ['mv', 'initrd', 'temp_media_dir/boot/x86_64/loader/initrd']
        )
        mock_size.assert_called_once_with(
            'temp_media_dir'
        )
        rootsize.accumulate_mbyte_file_sizes.assert_called_once_with()
        mock_isofs.assert_called_once_with(
            custom_args={
                'create_options': [
                    '-A', '0xffffffff',
                    '-p', '"KIWI - http://suse.github.com/kiwi"',
                    '-publisher', '"SUSE LINUX GmbH"',
                    '-V', '"volid"',
                    '-iso-level', '3', '-udf'
                ]
            }, device_provider=None, root_dir='temp_media_dir'
        )
        iso_image.create_on_file.assert_called_once_with(
            'target_dir/result-image.x86_64-1.2.3.iso'
        )
        mock_hybrid.assert_called_once_with(
            'offset', self.mbrid, 'target_dir/result-image.x86_64-1.2.3.iso'
        )
        assert self.result.add.call_args_list == [
            call(
                key='live_image',
                filename='target_dir/result-image.x86_64-1.2.3.iso',
                use_for_bundle=True,
                compress=False,
                shasum=True
            ),
            call(
                key='image_packages',
                filename='.packages',
                use_for_bundle=True,
                compress=False,
                shasum=False
            ),
            call(
                key='image_verified',
                filename='.verified',
                use_for_bundle=True,
                compress=False,
                shasum=False
            )
        ]
        self.setup.export_rpm_package_verification.assert_called_once_with(
            'target_dir'
        )
        self.setup.export_rpm_package_list.assert_called_once_with(
            'target_dir'
        )

    @patch('kiwi.builder.live.mkdtemp')
    @patch('kiwi.builder.live.Command.run')
    @raises(KiwiLiveBootImageError)
    def test_create_invalid_iso_structure(self, mock_command, mock_dtemp):
        self.live_image.live_type = 'bogus'
        self.live_image.create()

    @patch('shutil.copy')
    @patch('kiwi.builder.live.mkdtemp')
    @patch('kiwi.builder.live.Command.run')
    @patch('kiwi.builder.live.BootLoaderConfig')
    @patch_open
    @raises(KiwiLiveBootImageError)
    def test_create_no_kernel_found(
        self, mock_open, mock_boot, mock_command, mock_dtemp, mock_copy
    ):
        self.kernel.get_kernel.return_value = False
        self.live_image.create()

    @patch('shutil.copy')
    @patch('kiwi.builder.live.mkdtemp')
    @patch('kiwi.builder.live.Command.run')
    @patch('kiwi.builder.live.BootLoaderConfig')
    @patch_open
    @raises(KiwiLiveBootImageError)
    def test_create_no_hypervisor_found(
        self, mock_open, mock_boot, mock_command, mock_dtemp, mock_copy
    ):
        self.kernel.get_xen_hypervisor.return_value = False
        self.live_image.create()

    @patch('kiwi.builder.live.Path.wipe')
    def test_destructor(self, mock_wipe):
        self.live_image.media_dir = 'media-dir'
        self.live_image.__del__()
        assert mock_wipe.call_args_list == [
            call('media-dir')
        ]
        self.live_image.media_dir = None



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import re
import os

# project
from kiwi.command import Command
from kiwi.package_manager.base import PackageManagerBase
from kiwi.path import Path
from kiwi.exceptions import (
    KiwiRpmDatabaseReloadError,
    KiwiRequestError
)


class PackageManagerZypper(PackageManagerBase):
    """
    Implements base class for installation/deletion of
    packages and collections using zypper
    """
    def post_init(self, custom_args=None):
        """
        Post initialization method

        Store custom zypper arguments

        Attributes

        * :attr:`zypper_args`
            zypper arguments from repository runtime configuration

        * :attr:`command_env`
            zypper command environment from repository runtime configuration

        :param list custom_args: custom zypper arguments
        """
        self.custom_args = custom_args
        if not custom_args:
            self.custom_args = []

        runtime_config = self.repository.runtime_config()

        self.zypper_args = runtime_config['zypper_args']
        self.chroot_zypper_args = self.root_bind.move_to_root(
            self.zypper_args
        )

        self.command_env = runtime_config['command_env']
        self.chroot_command_env = dict(self.command_env)
        if 'ZYPP_CONF' in self.command_env:
            self.chroot_command_env['ZYPP_CONF'] = self.root_bind.move_to_root(
                [self.command_env['ZYPP_CONF']]
            )[0]

    def request_package(self, name):
        """
        Queue a package request

        :param string name: package name
        """
        self.package_requests.append(name)

    def request_collection(self, name):
        """
        Queue a collection request

        :param string name: zypper pattern name
        """
        self.collection_requests.append('pattern:' + name)

    def request_product(self, name):
        """
        Queue a product request

        :param string name: zypper product name
        """
        self.product_requests.append('product:' + name)

    def request_package_lock(self, name):
        """
        Queue a package lock(ignore) request

        :param string name: package name
        """
        self.lock_requests.append(name)

    def process_install_requests_bootstrap(self):
        """
        Process package install requests for bootstrap phase (no chroot)
        """
        command = ['zypper'] + self.zypper_args + [
            '--root', self.root_dir,
            'install', '--auto-agree-with-licenses'
        ] + self.custom_args + self._install_items()
        return Command.call(
            command, self.command_env
        )

    def process_install_requests(self):
        """
        Process package install requests for image phase (chroot)
        """
        if self.lock_requests:
            # Zypper supports the package lock via the zypper al command
            # This allows to block a package from being taken into
            # consideration when the dependency solver runs. However
            # the request might be ignored by the package manager if
            # the lock request conflicts with a required package
            # dependency. e.g setting a lock on the glibc package will
            # for sure not work in order to keep the system in a
            # consistent state. Thus such lock requests are primarily
            # for packages pulled in by a recommendation but not by a
            # hard requirement.
            lock_metadata_dir = ''.join([self.root_dir, '/etc/zypp'])
            if not os.path.exists(lock_metadata_dir):
                Path.create(lock_metadata_dir)
            for package in self.lock_requests:
                Command.run(
                    [
                        'chroot', self.root_dir, 'zypper'
                    ] + self.chroot_zypper_args + [
                        'al'
                    ] + self.custom_args + [package],
                    self.chroot_command_env
                )
        return Command.call(
            ['chroot', self.root_dir, 'zypper'] + self.chroot_zypper_args + [
                'install', '--auto-agree-with-licenses'
            ] + self.custom_args + self._install_items(),
            self.chroot_command_env
        )

    def process_delete_requests(self, force=False):
        """
        Process package delete requests (chroot)

        :param bool force: force deletion: true|false
        """
        delete_items = []
        for delete_item in self._delete_items():
            try:
                Command.run(['chroot', self.root_dir, 'rpm', '-q', delete_item])
                delete_items.append(delete_item)
            except Exception:
                # ignore packages which are not installed
                pass
        if not delete_items:
            raise KiwiRequestError(
                'None of the requested packages to delete are installed'
            )
        if force:
            force_options = ['--nodeps', '--allmatches', '--noscripts']
            return Command.call(
                [
                    'chroot', self.root_dir, 'rpm', '-e'
                ] + force_options + delete_items,
                self.chroot_command_env
            )
        else:
            return Command.call(
                [
                    'chroot', self.root_dir, 'zypper'
                ] + self.chroot_zypper_args + [
                    'remove', '-u', '--force-resolution'
                ] + delete_items,
                self.chroot_command_env
            )

    def update(self):
        """
        Process package update requests (chroot)
        """
        return Command.call(
            ['chroot', self.root_dir, 'zypper'] + self.chroot_zypper_args + [
                'update', '--auto-agree-with-licenses'
            ] + self.custom_args,
            self.chroot_command_env
        )

    def process_only_required(self):
        """
        Setup package processing only for required packages
        """
        if '--no-recommends' not in self.custom_args:
            self.custom_args.append('--no-recommends')

    def match_package_installed(self, package_name, zypper_output):
        """
        Match expression to indicate a package has been installed

        This match for the package to be installed in the output
        of the zypper command is not 100% accurate. There might
        be false positives due to sub package names starting with
        the same base package name

        :param list package_list: list of all packages
        :param string log_line: zypper status line
        """
        return re.match(
            '.*Installing: ' + re.escape(package_name) + '.*', zypper_output
        )

    def match_package_deleted(self, package_name, zypper_output):
        """
        Match expression to indicate a package has been deleted

        :param list package_list: list of all packages
        :param string log_line: zypper status line
        """
        return re.match(
            '.*Removing: ' + re.escape(package_name) + '.*', zypper_output
        )

    def database_consistent(self):
        """
        Check if rpm package database is consistent
        """
        try:
            Command.run(['chroot', self.root_dir, 'rpmdb', '--initdb'])
            return True
        except Exception:
            return False

    def dump_reload_package_database(self, version=45):
        """
        Dump and reload the rpm database to match the desired rpm db version
        Supported target rpm database db versions are

        * 45 (db45_load)
        * 48 (db48_load)

        :param string version: target rpm db version
        """
        db_load_for_version = {
            45: 'db45_load',
            48: 'db48_load'
        }
        if version not in db_load_for_version:
            raise KiwiRpmDatabaseReloadError(
                'Dump reload for rpm DB version: %s not supported' % version
            )
        if not self.database_consistent():
            reload_db_files = [
                '/var/lib/rpm/Name',
                '/var/lib/rpm/Packages'
            ]
            for db_file in reload_db_files:
                root_db_file = self.root_dir + db_file
                root_db_file_backup = root_db_file + '.bak'
                Command.run([
                    'db_dump', '-f', root_db_file_backup, root_db_file
                ])
                Command.run(['rm', '-f', root_db_file])
                Command.run([
                    db_load_for_version[version],
                    '-f', root_db_file_backup, root_db_file
                ])
                Command.run(['rm', '-f', root_db_file_backup])
            Command.run([
                'chroot', self.root_dir, 'rpm', '--rebuilddb'
            ])

    def _install_items(self):
        items = self.package_requests + self.collection_requests \
            + self.product_requests
        self.cleanup_requests()
        return items

    def _delete_items(self):
        # collections and products can't be deleted
        items = []
        items += self.package_requests
        self.cleanup_requests()
        return items



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#


class PackageManagerBase(object):
    """
    Implements base class for installation/deletion of
    packages and collections using a package manager

    Attributes

    * :attr:`repository`
        Instance of Repository

    * :attr:`root_dir`
        root directory path name

    * :attr:`root_bind`
        Instance of RootBind

    * :attr:`package_requests`
        list of packages to install or delete

    * :attr:`collection_requests`
        list of collections to install

    * :attr:`product_requests`
        list of products to install
    """
    def __init__(self, repository, custom_args=None):
        self.repository = repository
        self.root_dir = repository.root_dir
        self.root_bind = repository.root_bind
        self.package_requests = []
        self.collection_requests = []
        self.product_requests = []
        self.lock_requests = []

        self.post_init(custom_args)

    def post_init(self, custom_args=None):
        """
        Post initialization method

        Implementation in specialized package manager class

        :param list custom_args: unused
        """
        pass

    def request_package(self, name):
        """
        Queue a package request

        Implementation in specialized package manager class

        :param string name: unused
        """
        raise NotImplementedError

    def request_collection(self, name):
        """
        Queue a package collection

        Implementation in specialized package manager class

        :param string name: unused
        """
        raise NotImplementedError

    def request_product(self, name):
        """
        Queue a product request

        Implementation in specialized package manager class

        :param string name: unused
        """
        raise NotImplementedError

    def request_package_lock(self, name):
        """
        Queue a package lock(ignore) request

        Implementation in specialized package manager class

        :param string name: unused
        """
        raise NotImplementedError

    def process_install_requests_bootstrap(self):
        """
        Process package install requests for bootstrap phase (no chroot)

        Implementation in specialized package manager class
        """
        raise NotImplementedError

    def process_install_requests(self):
        """
        Process package install requests for image phase (chroot)

        Implementation in specialized package manager class
        """
        raise NotImplementedError

    def process_delete_requests(self, force=False):
        """
        Process package delete requests (chroot)

        Implementation in specialized package manager class

        :param bool force: unused
        """
        raise NotImplementedError

    def update(self):
        """
        Process package update requests (chroot)

        Implementation in specialized package manager class
        """
        raise NotImplementedError

    def process_only_required(self):
        """
        Setup package processing only for required packages

        Implementation in specialized package manager class
        """
        raise NotImplementedError

    def match_package_installed(self, package_list, log_line):
        """
        Match expression to indicate a package has been installed

        Implementation in specialized package manager class

        :param list package_list: unused
        :param string log_line: unused
        """
        raise NotImplementedError

    def match_package_deleted(self, package_list, log_line):
        """
        Match expression to indicate a package has been deleted

        Implementation in specialized package manager class

        :param list package_list: unused
        :param string log_line: unused
        """
        raise NotImplementedError

    def database_consistent(self):
        """
        Check if package database is consistent

        Implementation in specialized package manager class
        """
        raise NotImplementedError

    def dump_reload_package_database(self, version=45):
        """
        For rpm based package managers, dump and reload the database
        to match the desired rpm db version

        Implementation in specialized package manager class

        :param string version: unused
        """
        raise NotImplementedError

    def cleanup_requests(self):
        """
        Cleanup request queues
        """
        del self.package_requests[:]
        del self.collection_requests[:]
        del self.product_requests[:]
        del self.lock_requests[:]



import sys
import mock

import kiwi

from .test_helper import argv_kiwi_tests

from kiwi.tasks.system_create import SystemCreateTask


class TestSystemCreateTask(object):
    def setup(self):
        sys.argv = [
            sys.argv[0], '--profile', 'vmxFlavour', 'system', 'create',
            '--root', '../data/root-dir', '--target-dir', 'some-target'
        ]
        kiwi.tasks.system_create.Privileges = mock.Mock()
        kiwi.tasks.system_create.Path = mock.Mock()

        kiwi.tasks.system_create.Help = mock.Mock(
            return_value=mock.Mock()
        )

        self.runtime_checker = mock.Mock()
        kiwi.tasks.base.RuntimeChecker = mock.Mock(
            return_value=self.runtime_checker
        )

        self.setup = mock.Mock()
        kiwi.tasks.system_create.SystemSetup = mock.Mock(
            return_value=self.setup
        )

        self.result = mock.Mock()
        self.builder = mock.MagicMock()
        self.builder.create = mock.Mock(
            return_value=self.result
        )
        kiwi.tasks.system_create.ImageBuilder = mock.Mock(
            return_value=self.builder
        )

        self.task = SystemCreateTask()

    def teardown(self):
        sys.argv = argv_kiwi_tests

    def _init_command_args(self):
        self.task.command_args = {}
        self.task.command_args['help'] = False
        self.task.command_args['create'] = False
        self.task.command_args['--root'] = '../data/root-dir'
        self.task.command_args['--target-dir'] = 'some-target'

    def test_process_system_create(self):
        self._init_command_args()
        self.task.command_args['create'] = True
        self.task.process()
        self.runtime_checker.check_target_directory_not_in_shared_cache.called_once_with('some-target')
        self.setup.call_image_script.assert_called_once_with()
        self.builder.create.assert_called_once_with()
        self.result.print_results.assert_called_once_with()
        self.result.dump.assert_called_once_with(
            'some-target/kiwi.result'
        )

    def test_process_system_create_help(self):
        self._init_command_args()
        self.task.command_args['help'] = True
        self.task.command_args['create'] = True
        self.task.process()
        self.task.manual.show.assert_called_once_with(
            'kiwi::system::create'
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import platform
from collections import OrderedDict
from collections import namedtuple
from tempfile import NamedTemporaryFile

# project
from kiwi.system.uri import Uri
from kiwi.repository import Repository
from kiwi.system.root_bind import RootBind
from kiwi.system.root_init import RootInit
from kiwi.command import Command
from kiwi.command_process import CommandProcess
from kiwi.utils.sync import DataSync
from kiwi.logger import log
from kiwi.defaults import Defaults
from kiwi.system.users import Users
from kiwi.system.shell import Shell
from kiwi.path import Path
from kiwi.archive.tar import ArchiveTar
from kiwi.utils.compress import Compress

from kiwi.exceptions import (
    KiwiImportDescriptionError,
    KiwiScriptFailed
)


class SystemSetup(object):
    """
    Implementation of system setup steps supported by kiwi.
    kiwi is not responsible for the system configuration, however
    some setup steps needs to be performed in order to provide
    a minimal work environment inside of the image according to
    the desired image type.

    Attributes

    * :attr:`arch`
        platform.machine
        The 32bit x86 platform is handled as 'ix86'

    * :attr:`xml_state`
        Instance of XMLState

    * :attr:`description_dir`
        path to image description directory

    * :attr:`derived_description_dir`
        path to derived_description_dir
        boot image descriptions inherits data from the system image
        description, thus they are derived from another image
        description directory which is needed to e.g find system
        image archives, overlay files

    * :attr:`root_dir`
        root directory path name
    """
    def __init__(self, xml_state, root_dir):
        self.arch = platform.machine()
        if self.arch == 'i686' or self.arch == 'i586':
            self.arch = 'ix86'
        self.xml_state = xml_state
        self.description_dir = \
            xml_state.xml_data.description_dir
        self.derived_description_dir = \
            xml_state.xml_data.derived_description_dir
        self.root_dir = root_dir
        self._preferences_lookup()
        self._oemconfig_lookup()

    def import_description(self):
        """
        Import XML descriptions, custom scripts, archives and
        script helper methods
        """
        log.info('Importing Image description to system tree')
        description = self.root_dir + '/image/config.xml'
        log.info('--> Importing state XML description as image/config.xml')
        Path.create(self.root_dir + '/image')
        with open(description, 'w') as config:
            config.write('<?xml version="1.0" encoding="utf-8"?>')
            self.xml_state.xml_data.export(outfile=config, level=0)

        self._import_custom_scripts()
        self._import_custom_archives()

    def cleanup(self):
        """
        Delete all traces of a kiwi description which are not
        required in the later image
        """
        Command.run(['rm', '-r', '-f', '/.kconfig', '/image'])

    def import_repositories_marked_as_imageinclude(self):
        """
        Those <repository> sections which are marked with the
        imageinclude attribute should be permanently added to
        the image repository configuration
        """
        repository_sections = self.xml_state.get_repository_sections()
        root = RootInit(
            root_dir=self.root_dir, allow_existing=True
        )
        repo = Repository(
            RootBind(root), self.xml_state.get_package_manager()
        )
        repo.use_default_location()
        repo.delete_all_repos()
        for xml_repo in repository_sections:
            repo_marked_for_image_include = xml_repo.get_imageinclude()

            if repo_marked_for_image_include:
                repo_type = xml_repo.get_type()
                repo_source = xml_repo.get_source().get_path()
                repo_user = xml_repo.get_username()
                repo_secret = xml_repo.get_password()
                repo_alias = xml_repo.get_alias()
                repo_priority = xml_repo.get_priority()
                repo_dist = xml_repo.get_distribution()
                repo_components = xml_repo.get_components()
                uri = Uri(repo_source, repo_type)
                repo_source_translated = uri.translate()
                if not repo_alias:
                    repo_alias = uri.alias()
                log.info('Setting up image repository %s', repo_source)
                log.info('--> Type: %s', repo_type)
                log.info('--> Translated: %s', repo_source_translated)
                log.info('--> Alias: %s', repo_alias)
                repo.add_repo(
                    repo_alias, repo_source_translated,
                    repo_type, repo_priority, repo_dist, repo_components,
                    repo_user, repo_secret, uri.credentials_file_name()
                )

    def import_shell_environment(self, profile):
        """
        Create profile environment to let scripts consume
        information from the XML description.

        :param object profile: Instance of Profile
        """
        profile_file = self.root_dir + '/.profile'
        log.info('Creating .profile environment')
        profile_environment = profile.create()
        with open(profile_file, 'w') as profile:
            for line in profile_environment:
                profile.write(line + '\n')
                log.debug('--> %s', line)

    def import_overlay_files(
        self, follow_links=False, preserve_owner_group=False
    ):
        """
        Copy overlay files from the image description to
        the image root tree. Supported are a root/ directory
        or a root.tar.gz tarball. The root/ directory takes
        precedence over the tarball

        :param bool follow_links: follow symlinks true|false
        :param bool preserve_owner_group: preserve permissions true|false
        """
        overlay_directory = self.description_dir + '/root/'
        overlay_archive = self.description_dir + '/root.tar.gz'
        if os.path.exists(overlay_directory):
            log.info('Copying user defined files to image tree')
            sync_options = [
                '-r', '-p', '-t', '-D', '-H', '-X', '-A', '--one-file-system'
            ]
            if follow_links:
                sync_options.append('--copy-links')
            else:
                sync_options.append('--links')
            if preserve_owner_group:
                sync_options.append('-o')
                sync_options.append('-g')
            data = DataSync(
                overlay_directory, self.root_dir
            )
            data.sync_data(
                options=sync_options
            )
        elif os.path.exists(overlay_archive):
            log.info('Extracting user defined files from archive to image tree')
            archive = ArchiveTar(overlay_archive)
            archive.extract(self.root_dir)

    def setup_hardware_clock(self):
        """
        Setup etc/adjtime by running hwclock
        """
        if 'hwclock' in self.preferences:
            log.info(
                'Setting up hardware clock: %s', self.preferences['hwclock']
            )
            Command.run([
                'chroot', self.root_dir,
                'hwclock', '--adjust', '--' + self.preferences['hwclock']
            ])

    def setup_keyboard_map(self):
        """
        Setup etc/sysconfig/keyboard console keyboard
        """
        if 'keytable' in self.preferences:
            keyboard_config = self.root_dir + '/etc/sysconfig/keyboard'
            if os.path.exists(keyboard_config):
                log.info(
                    'Setting up keytable: %s', self.preferences['keytable']
                )
                Shell.run_common_function(
                    'baseUpdateSysConfig', [
                        keyboard_config, 'KEYTABLE',
                        '"' + self.preferences['keytable'] + '"'
                    ]
                )
            else:
                log.warning(
                    'keyboard setup skipped etc/sysconfig/keyboard not found'
                )

    def setup_locale(self):
        """
        Setup etc/sysconfig/language UTF8 locale
        """
        if 'locale' in self.preferences:
            lang_config = self.root_dir + '/etc/sysconfig/language'
            if os.path.exists(lang_config):
                log.info(
                    'Setting up locale: %s', self.preferences['locale']
                )
                Shell.run_common_function(
                    'baseUpdateSysConfig', [
                        lang_config, 'RC_LANG',
                        self.preferences['locale'].split(',')[0] + '.UTF-8'
                    ]
                )
            else:
                log.warning(
                    'locale setup skipped etc/sysconfig/language not found'
                )

    def setup_timezone(self):
        """
        Setup timezone symlink
        """
        if 'timezone' in self.preferences:
            log.info(
                'Setting up timezone: %s', self.preferences['timezone']
            )
            zoneinfo = '/usr/share/zoneinfo/' + self.preferences['timezone']
            Command.run([
                'chroot', self.root_dir,
                'ln', '-s', '-f', zoneinfo, '/etc/localtime'
            ])

    def setup_groups(self):
        """
        Add groups for configured users
        """
        system_users = Users(self.root_dir)

        for user in self.xml_state.get_users():
            for group in self.xml_state.get_user_groups(user.get_name()):
                if not system_users.group_exists(group):
                    log.info('Adding group %s', group)
                    system_users.group_add(group, [])

    def setup_users(self):
        """
        Add/Modify configured users
        """
        system_users = Users(self.root_dir)

        for user in self.xml_state.get_users():
            log.info('Setting up user %s', user.get_name())
            password = user.get_password()
            password_format = user.get_pwdformat()
            home_path = user.get_home()
            user_name = user.get_name()
            user_id = user.get_id()
            user_realname = user.get_realname()
            user_shell = user.get_shell()
            user_groups = self.xml_state.get_user_groups(user.get_name())

            user_exists = system_users.user_exists(user_name)

            options = []
            if password_format == 'plain':
                password = self._create_passwd_hash(password)
            if password:
                options.append('-p')
                options.append(password)
            if user_shell:
                options.append('-s')
                options.append(user_shell)
            if len(user_groups):
                options.append('-g')
                options.append(user_groups[0])
                if len(user_groups) > 1:
                    options.append('-G')
                    options.append(','.join(user_groups[1:]))
            if user_id:
                options.append('-u')
                options.append('{0}'.format(user_id))
            if user_realname:
                options.append('-c')
                options.append(user_realname)
            if not user_exists and home_path:
                options.append('-m')
                options.append('-d')
                options.append(home_path)

            if user_exists:
                log.info(
                    '--> Modifying user: %s [%s]',
                    user_name,
                    user_groups[0] if len(user_groups) else ''
                )
                system_users.user_modify(user_name, options)
            else:
                log.info(
                    '--> Adding user: %s [%s]',
                    user_name,
                    user_groups[0] if len(user_groups) else ''
                )
                system_users.user_add(user_name, options)
                if home_path:
                    log.info(
                        '--> Setting permissions for %s', home_path
                    )
                    # Emtpy group string assumes the login or default group
                    system_users.setup_home_for_user(
                        user_name,
                        user_groups[0] if len(user_groups) else '',
                        home_path
                    )

    def import_image_identifier(self):
        """
        Create etc/ImageID identifier file
        """
        image_id = self.xml_state.xml_data.get_id()
        if image_id and os.path.exists(self.root_dir + '/etc'):
            image_id_file = self.root_dir + '/etc/ImageID'
            log.info('Creating identifier: %s as %s', image_id, image_id_file)
            with open(image_id_file, 'w') as identifier:
                identifier.write('%s\n' % image_id)

    def set_selinux_file_contexts(self, security_context_file):
        """
        Initialize the security context fields (extended attributes)
        on the files matching the security_context_file

        :param string security_context_file: path file name
        """
        log.info('Processing SELinux file security contexts')
        Command.run(
            [
                'chroot', self.root_dir,
                'setfiles', security_context_file, '/', '-v'
            ]
        )

    def export_modprobe_setup(self, target_root_dir):
        """
        Export etc/modprobe.d to given root_dir

        :param string target_root_dir: path name
        """
        modprobe_config = self.root_dir + '/etc/modprobe.d'
        if os.path.exists(modprobe_config):
            log.info('Export modprobe configuration')
            Path.create(target_root_dir + '/etc')
            data = DataSync(
                modprobe_config, target_root_dir + '/etc/'
            )
            data.sync_data(
                options=['-z', '-a']
            )

    def export_rpm_package_list(self, target_dir):
        """
        Export image rpm package list as metadata reference
        used by the open buildservice

        :param string target_dir: path name
        """
        if os.path.exists(self.root_dir + '/var/lib/rpm/Packages'):
            log.info('Export rpm packages metadata')
            filename = ''.join(
                [
                    target_dir, '/',
                    self.xml_state.xml_data.get_name(),
                    '.' + self.arch,
                    '-' + self.xml_state.get_image_version(),
                    '.packages'
                ]
            )
            query_call = Command.run(
                [
                    'rpm', '--root', self.root_dir, '-qa', '--qf',
                    '|'.join(
                        [
                            '%{NAME}', '%{EPOCH}', '%{VERSION}', '%{RELEASE}',
                            '%{ARCH}', '%{DISTURL}', '\\n'
                        ]
                    )
                ]
            )
            with open(filename, 'w') as packages:
                packages.write(query_call.output)
            return filename

    def export_rpm_package_verification(self, target_dir):
        """
        Export rpm package verification result as metadata reference
        used by the open buildservice

        :param string target_dir: path name
        """
        if os.path.exists(self.root_dir + '/var/lib/rpm/Packages'):
            log.info('Export rpm verification metadata')
            filename = ''.join(
                [
                    target_dir, '/',
                    self.xml_state.xml_data.get_name(),
                    '.' + self.arch,
                    '-' + self.xml_state.get_image_version(),
                    '.verified'
                ]
            )
            query_call = Command.run(
                command=['rpm', '--root', self.root_dir, '-Va'],
                raise_on_error=False
            )
            with open(filename, 'w') as verified:
                verified.write(query_call.output)
            return filename

    def call_config_script(self):
        """
        Call config.sh script chrooted
        """
        self._call_script('config.sh')

    def call_image_script(self):
        """
        Call images.sh script chrooted
        """
        self._call_script('images.sh')

    def call_edit_boot_config_script(self, filesystem, boot_part_id):
        """
        Call configured editbootconfig script _NON_ chrooted

        Pass the boot filesystem name and the partition number of
        the boot partition as parameters to the call

        :param string filesystem: boot filesystem name
        :param int boot_part_id: boot partition number
        """
        self._call_script_no_chroot(
            'edit_boot_config.sh', [filesystem, format(boot_part_id)]
        )

    def call_edit_boot_install_script(self, diskname, boot_device_node):
        """
        Call configured editbootinstall script _NON_ chrooted

        Pass the disk file name and the device node of the boot partition
        as parameters to the call

        :param string diskname: file path name
        :param string boot_device_node: boot device node name
        """
        self._call_script_no_chroot(
            'edit_boot_install.sh', [diskname, boot_device_node]
        )

    def create_fstab(self, entries):
        """
        Create etc/fstab from given list of entries
        """
        fstab_file = self.root_dir + '/etc/fstab'
        with open(fstab_file, 'w') as fstab:
            for entry in entries:
                fstab.write(entry + os.linesep)

    def create_init_link_from_linuxrc(self):
        """
        kiwi boot images provides the linuxrc script, however the kernel
        also expects an init executable to be present. This method creates
        a hard link to the linuxrc file
        """
        Command.run(
            ['ln', self.root_dir + '/linuxrc', self.root_dir + '/init']
        )

    def create_recovery_archive(self):
        """
        Create a compressed recovery archive from the root tree
        for use with kiwi's recvoery system. The method creates
        additional data into the image root filesystem which is
        deleted prior to the creation of a new recovery data set
        """
        # cleanup
        bash_comand = [
            'rm', '-f', self.root_dir + '/recovery.*'
        ]
        Command.run(['bash', '-c', ' '.join(bash_comand)])
        if not self.oemconfig['recovery']:
            return
        # recovery.tar
        log.info('Creating recovery tar archive')
        metadata = {
            'archive_name':
                self.root_dir + '/recovery.tar',
            'archive_filecount':
                self.root_dir + '/recovery.tar.files',
            'archive_size':
                self.root_dir + '/recovery.tar.size',
            'partition_size':
                self.root_dir + '/recovery.partition.size',
            'partition_filesystem':
                self.root_dir + '/recovery.tar.filesystem'
        }
        recovery_archive = NamedTemporaryFile(
            delete=False
        )
        archive = ArchiveTar(
            filename=recovery_archive.name,
            create_from_file_list=False
        )
        archive.create(
            source_dir=self.root_dir,
            exclude=['dev', 'proc', 'sys'],
            options=[
                '--numeric-owner',
                '--hard-dereference',
                '--preserve-permissions'
            ]
        )
        Command.run(
            ['mv', recovery_archive.name, metadata['archive_name']]
        )
        # recovery.tar.filesystem
        recovery_filesystem = self.xml_state.build_type.get_filesystem()
        with open(metadata['partition_filesystem'], 'w') as partfs:
            partfs.write('%s' % recovery_filesystem)
        log.info(
            '--> Recovery partition filesystem: %s', recovery_filesystem
        )
        # recovery.tar.files
        bash_comand = [
            'tar', '-tf', metadata['archive_name'], '|', 'wc', '-l'
        ]
        tar_files_call = Command.run(
            ['bash', '-c', ' '.join(bash_comand)]
        )
        tar_files_count = int(tar_files_call.output.rstrip('\n'))
        with open(metadata['archive_filecount'], 'w') as files:
            files.write('%d\n' % tar_files_count)
        log.info(
            '--> Recovery file count: %d files', tar_files_count
        )
        # recovery.tar.size
        recovery_archive_size_bytes = os.path.getsize(metadata['archive_name'])
        with open(metadata['archive_size'], 'w') as size:
            size.write('%d' % recovery_archive_size_bytes)
        log.info(
            '--> Recovery uncompressed size: %d mbytes',
            int(recovery_archive_size_bytes / 1048576)
        )
        # recovery.tar.gz
        log.info('--> Compressing recovery archive')
        compress = Compress(self.root_dir + '/recovery.tar')
        compress.gzip()
        # recovery.partition.size
        recovery_archive_gz_size_mbytes = int(
            os.path.getsize(metadata['archive_name'] + '.gz') / 1048576
        )
        recovery_partition_mbytes = recovery_archive_gz_size_mbytes \
            + Defaults.get_recovery_spare_mbytes()
        with open(metadata['partition_size'], 'w') as gzsize:
            gzsize.write('%d' % recovery_partition_mbytes)
        log.info(
            '--> Recovery partition size: %d mbytes',
            recovery_partition_mbytes
        )
        # delete recovery archive if inplace recovery is requested
        # In this mode the recovery archive is created at install time
        # and not at image creation time. However the recovery metadata
        # is preserved in order to be able to check if enough space
        # is available on the disk to create the recovery archive.
        if self.oemconfig['recovery_inplace']:
            log.info(
                '--> Inplace recovery requested, deleting archive'
            )
            Path.wipe(metadata['archive_name'] + '.gz')

    def _import_custom_archives(self):
        """
        Import custom tar archive files
        """
        archive_list = []
        system_archives = self.xml_state.get_system_archives()
        bootstrap_archives = self.xml_state.get_bootstrap_archives()
        if system_archives:
            archive_list += system_archives
        if bootstrap_archives:
            archive_list += bootstrap_archives

        description_target = self.root_dir + '/image/'

        for archive in archive_list:
            archive_is_absolute = archive.startswith('/')
            if archive_is_absolute:
                archive_file = archive
            else:
                archive_file = self.description_dir + '/' + archive

            archive_exists = os.path.exists(archive_file)

            if not archive_exists:
                if self.derived_description_dir and not archive_is_absolute:
                    archive_file = self.derived_description_dir + '/' + archive
                    archive_exists = os.path.exists(archive_file)

            if archive_exists:
                log.info(
                    '--> Importing %s archive as %s',
                    archive_file, 'image/' + archive
                )
                Command.run(
                    ['cp', archive_file, description_target]
                )
            else:
                raise KiwiImportDescriptionError(
                    'Specified archive %s does not exist' % archive_file
                )

    def _import_custom_scripts(self):
        """
        Import custom scripts
        """
        # custom_scripts defines a dictionary with all script hooks
        # for each script name a the filepath and additional flags
        # are defined. the filepath could be either a relative or
        # absolute information. If filepath is set to None this indicates
        # the script hook is not used. The raise_if_not_exists flag
        # causes kiwi to raise an exception if a specified script
        # filepath does not exist
        script_type = namedtuple(
            'script_type', ['filepath', 'raise_if_not_exists']
        )
        custom_scripts = {
            'config.sh': script_type(
                filepath='config.sh',
                raise_if_not_exists=False
            ),
            'images.sh': script_type(
                filepath='images.sh',
                raise_if_not_exists=False
            ),
            'edit_boot_config.sh': script_type(
                filepath=self.xml_state.build_type.get_editbootconfig(),
                raise_if_not_exists=True
            ),
            'edit_boot_install.sh': script_type(
                filepath=self.xml_state.build_type.get_editbootinstall(),
                raise_if_not_exists=True
            )
        }
        sorted_custom_scripts = OrderedDict(
            sorted(custom_scripts.items())
        )

        description_target = self.root_dir + '/image/'
        need_script_helper_functions = False

        for name, script in list(sorted_custom_scripts.items()):
            if script.filepath:
                if script.filepath.startswith('/'):
                    script_file = script.filepath
                else:
                    script_file = self.description_dir + '/' + script.filepath
                if os.path.exists(script_file):
                    log.info(
                        '--> Importing %s script as %s',
                        script.filepath, 'image/' + name
                    )
                    Command.run(
                        ['cp', script_file, description_target + name]
                    )
                    need_script_helper_functions = True
                elif script.raise_if_not_exists:
                    raise KiwiImportDescriptionError(
                        'Specified script %s does not exist' % script_file
                    )

        if need_script_helper_functions:
            log.info('--> Importing script helper functions')
            Command.run(
                [
                    'cp',
                    Defaults.get_common_functions_file(),
                    self.root_dir + '/.kconfig'
                ]
            )

    def _call_script(self, name):
        if os.path.exists(self.root_dir + '/image/' + name):
            config_script = Command.call(
                ['chroot', self.root_dir, 'bash', '/image/' + name]
            )
            process = CommandProcess(
                command=config_script, log_topic='Calling ' + name + ' script'
            )
            result = process.poll_and_watch()
            if result.returncode != 0:
                raise KiwiScriptFailed(
                    '%s failed: %s' % (name, format(result.stderr))
                )

    def _call_script_no_chroot(self, name, option_list):
        if os.path.exists(self.root_dir + '/image/' + name):
            bash_command = [
                'cd', self.root_dir, '&&',
                'bash', '--norc', 'image/' + name, ' '.join(option_list)
            ]
            config_script = Command.call(
                ['bash', '-c', ' '.join(bash_command)]
            )
            process = CommandProcess(
                command=config_script, log_topic='Calling ' + name + ' script'
            )
            result = process.poll_and_watch()
            if result.returncode != 0:
                raise KiwiScriptFailed(
                    '%s failed: %s' % (name, format(result.stderr))
                )

    def _create_passwd_hash(self, password):
        openssl = Command.run(
            ['openssl', 'passwd', '-1', '-salt', 'xyz', password]
        )
        return openssl.output[:-1]

    def _preferences_lookup(self):
        self.preferences = {}
        for preferences in self.xml_state.get_preferences_sections():
            timezone_section = preferences.get_timezone()
            locale_section = preferences.get_locale()
            hwclock_section = preferences.get_hwclock()
            keytable_section = preferences.get_keytable()
            if timezone_section:
                self.preferences['timezone'] = timezone_section[0]
            if locale_section:
                self.preferences['locale'] = locale_section[0]
            if hwclock_section:
                self.preferences['hwclock'] = hwclock_section[0]
            if keytable_section:
                self.preferences['keytable'] = keytable_section[0]

    def _oemconfig_lookup(self):
        self.oemconfig = {
            'recovery_inplace': False,
            'recovery': False
        }
        oemconfig = self.xml_state.get_build_type_oemconfig_section()
        if oemconfig:
            self.oemconfig['recovery'] = \
                self._text(oemconfig.get_oem_recovery())
            self.oemconfig['recovery_inplace'] = \
                self._text(oemconfig.get_oem_inplace_recovery())

    def _text(self, section_content):
        """
        Helper method to return the text for XML elements of the
        following structure: <section>text</section>.
        """
        if section_content:
            return section_content[0]



from mock import patch

import mock

from kiwi.filesystem.setup import FileSystemSetup


class TestFileSystemSetup(object):
    @patch('kiwi.filesystem.setup.SystemSize')
    def setup(self, mock_size):
        size = mock.Mock()
        size.accumulate_mbyte_file_sizes = mock.Mock(
            return_value=42
        )
        size.customize = mock.Mock(
            return_value=42
        )
        mock_size.return_value = size
        self.xml_state = mock.Mock()
        self.xml_state.get_build_type_name = mock.Mock(
            return_value='ext4'
        )
        self.setup = FileSystemSetup(
            self.xml_state, 'root_dir'
        )

    def test_setup_with_pxe_type(self):
        self.xml_state.get_build_type_name = mock.Mock(
            return_value='pxe'
        )
        self.xml_state.build_type.get_filesystem = mock.Mock(
            return_value='xfs'
        )
        setup = FileSystemSetup(
            self.xml_state, 'root_dir'
        )
        assert setup.requested_filesystem == 'xfs'

    def test_get_size_mbytes_calculated(self):
        self.setup.configured_size = None
        assert self.setup.get_size_mbytes() == 42

    def test_get_size_mbytes_configured_additive(self):
        self.setup.configured_size.mbytes = 20
        self.setup.configured_size.additive = True
        assert self.setup.get_size_mbytes() == 62

    @patch('kiwi.logger.log.warning')
    def test_get_size_mbytes_configured(self, mock_log_warn):
        self.setup.configured_size.mbytes = 3
        self.setup.configured_size.additive = False
        assert self.setup.get_size_mbytes() == 3
        assert mock_log_warn.called
